/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "classes/unCore/unBeep/UnBeep.ctl"

class UnBeepCPC6:UnBeep
{

  protected void ConnectDPECallback() synchronized(m_connectedDPEs)
  {
    for (int i = 1; i <= dynlen(m_deviceList); i++) {
      if (!CheckDeviceDPE(m_deviceList[i])) break;

      if (!dynContains(m_connectedDPEs,  m_deviceList[i] + "ProcessInput.StsReg01")) {
          dynAppend(m_connectedDPEs, m_deviceList[i] + "ProcessInput.StsReg01");
          dynAppend(m_connectedDPEs, m_deviceList[i] + "ProcessInput.StsReg01:_online.._invalid");
          int rc = dpConnect("ValuesChangedCB", m_deviceList[i] + "ProcessInput.StsReg01",
                                                m_deviceList[i] + "ProcessInput.StsReg01:_online.._invalid");
          dyn_errClass err = getLastError();
          if (dynlen(err)) fwException_raise(excInfo, "ERROR",
                                            "UnBeepCPC6 could not dpConnect to devices " + m_deviceList[i], "");
      }
    }

    if (dynlen(excInfo)) {
      dynClear(m_systemNameList);
      DisconnectDPECallback();
      setAlarmState(UnBeepAlarmState::ConfigurationError);
    }

    checkHandleExceptions();
  }


  // This function is called whenever there is a value change in any of the registered dpe for the
  // unBeepCPC6. However, if more than one alias/device has been specified for the unBeepCPC6 in the Application
  // Configuration, it is necessary to obtain the values of all the devices/alias dpes to obtain the
  // overrall status of the unBeepCPC6. Hence, the input parameter, pair dpe-value, that has changed and has
  // triggered the ValuesChangedCB function, is never used, since all the devices/alias dpes need to be
  // reevaluated. In summary, the input parameters are just used as trigger to call the function ValuesChangedCB
  // to check the alias/devices specified for the unBeepCPC6.
  protected void ValuesChangedCB(dyn_string notUsedDpe, dyn_mixed notUsedVal) synchronized(m_connectedDPEs)
  {
    mixed val;
    dyn_string dpes;
    dyn_mixed values;

    for (int i = 1; i <= dynlen(m_connectedDPEs); i++) {
        if (!CheckDeviceDPE(m_connectedDPEs[i])) break;

        int rc = dpGet(m_connectedDPEs[i], val);
        dyn_errClass err = getLastError();
        if (dynlen(err)) {
            fwException_raise(excInfo, "ERROR",
                             "UnBeepCPC6 could not dpGet from datapoint element " + m_connectedDPEs[i], "");
            break;
        }

        dynAppend(dpes, m_connectedDPEs[i]);
        dynAppend(values, val);
    }

    // m_connectedDPEs length might be 0 if it was cleared due to an exception raised
    if (dynlen(excInfo) || dynlen(dpes) == 0) {
      dynClear(m_systemNameList);
      DisconnectDPECallback();
      setAlarmState(UnBeepAlarmState::ConfigurationError);
      checkHandleExceptions();
      return;
    }

    UnBeepAlarmState state = UnBeepAlarmState::OK;

    // As a callback parameters we have pairs of DPE:_online.._value and DPE:_online.._invalid
    if (dynlen(dpes) % 2 != 0 || dynlen(dpes) == 0) {
      state = UnBeepAlarmState::NotConfigured;
    } else {
      for (int i = 1; i <= dynlen(dpes); i++) {
        // let's see all the :_online.._invalid
        if (i % 2 == 0 && values[i] == TRUE) {
            state = UnBeepAlarmState::InvalidData;
            break;
        }
        if (i % 2 != 0) {
          if      (getBit(values[i], UN_STSREG01_ALMSKST) && (int) state < (int) UnBeepAlarmState::AlarmMasked) state = UnBeepAlarmState::AlarmMasked;
          else if (getBit(values[i], UN_STSREG01_ALUNACK) && (int) state < (int) UnBeepAlarmState::AlarmNotAck) state = UnBeepAlarmState::AlarmNotAck;
          else if (getBit(values[i], UN_STSREG01_POSST)   && (int) state < (int) UnBeepAlarmState::AlarmActive) state = UnBeepAlarmState::AlarmActive;
        }
      }
    }

    setAlarmState(state);       // this already triggers the redraw
  }


  protected bool CheckDeviceDPE(string device) {
    if (!dpExists(device)) {
      fwException_raise(excInfo, "ERROR",
                        "UnBeepCPC6: device " + device +
                        " does not exist", "");
      return false;
    } else if (dpTypeName(device)!="CPC_AnalogAlarm"  &&
               dpTypeName(device)!="CPC_DigitalAlarm" &&
               dpTypeName(device)!="Alarm") {
      fwException_raise(excInfo, "ERROR",
                        "UnBeepCPC6: device " + unGenericDpFunctions_getSystemName(device) +
                        dpGetAlias(device) + " (" + device +
                        ") is of unsupported type " + dpTypeName(device) +
                        ". Allowed CPC_AnalogAlarm, CPC_DigitalAlarm, Alarm(CPC5 compat)", "");
      return false;
    }
    return true;
  }
};
