/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "classes/fwTree/FwTree.ctl"
#uses "classes/unTree/fileParsers/unTreeFileParser.ctl"

//assuming device name -> actual value
//node name -> just name displayed in tree

class UnTreeDeviceParser {
    protected shared_ptr<UnTreeProcessContext> _context = nullptr;
    protected string _device;
    protected shared_ptr<mapping> _data = nullptr;

    public UnTreeDeviceParser(const anytype device, shared_ptr<UnTreeProcessContext> context = nullptr){
        _context = context;
        if(_context == nullptr){
            _context = new UnTreeProcessContext();
        }
        _device = device;
    }

    public string getDeviceDp(){
        return _device;
    }

    public bool hasExistingDp(){
        return dpExists(_device);
    }

    // public bool matchesWithDP(){
    //     if(!hasExistingDp()){
    //         return false;
    //     }
    //     return true; // to be checked with details;
    // }

    public void exportToFile(shared_ptr<UnTreeFileParser> fileParser){
    // to be overwriten
    }

    // public int getStatus(){
    //     try { this._assertData(); } catch { return NODE_STATUS_ERR; }
    //     if(_device == "" || !this.hasExistingDp()){ return NODE_STATUS_NEW; }
    //     return this.matchesWithDP() ? NODE_STATUS_OK : NODE_STATUS_DIFF;
    // }

    public bool isDeviceOk(const string deviceType = "", const bool shouldTrow = true){
        const string device = this.getDeviceDp();
        if(!dpExists(device)){
            if(shouldTrow){
                FwException::raise("Device/DP does not exist: " + device);
            }
            return false;
        }
        if(deviceType != ""){
            const string tp = dpTypeName(device);
            if(tp != deviceType){
                if(shouldTrow){
                    FwException::raise("Device/DP type wrong (" + tp + ") vs (expected:" + deviceType + ") for device:" + device);
                }
                return false;
            }
        }
        return true;
    }

    public anytype encodeUserData(anytype userData) {
        return userData;
    }

    public anytype decodeUserData(anytype userData) {
        return userData;
    }

    public void exportNavigationDevices(shared_ptr<UnTreeFileParser> fileParser){
        const  string device = this.getDeviceDp();
        dyn_string panelLabel, panelNameList, panelDollarParam, exceptionInfo;
        unPanel_getPanelList(device, panelLabel, panelNameList, panelDollarParam, exceptionInfo);
        FwException::checkRaise(exceptionInfo);
        fileParser.addNode(NODE_PARAMNAME_NAVIGATION);
        for(int i=1; i<=c_unPanelConfiguration_maxPanelNavigation; i++) {
            fileParser.addValue(
                "link"+i,
                makeDynString(
                    panelLabel[i],
                    _context.encodeDeviceName(panelNameList[i]),
                    panelDollarParam[i]
                ),
                -1,
                c_panel_navigation_delimiter
            );
            _context.addDevice(UnTreeProcessorParam::MISSING_CANDIDATES,panelNameList[i]);
        }
    }

    public void setNavigation(const string dp){
        FwException::assert(dpExists(dp), "Could not set navigation of non-existent dp: " + dp);
        FwException::assertNotNull(_data, "Could not extract navigation from empty data");
        FwException::assert(_data.contains(NODE_PARAMNAME_NAVIGATION), "Navigation data missing");
        dyn_string labels, names, dollars;
		for(int i=1; i<=c_unPanelConfiguration_maxPanelNavigation; i++) {
            string sLink;
			sprintf(sLink, "link%d", i);
            if(!_data[NODE_PARAMNAME_NAVIGATION].contains(sLink)){
                continue;
            }
			dyn_string data = strsplit(_data[NODE_PARAMNAME_NAVIGATION][sLink],"|");
			labels[i] = ( data.count()>0 ? data.at(0) : "" );
			names[i] = _context.decodeDeviceName( data.count()>1 ? data.at(1) : "" );
			dollars[i] = ( data.count()>2 ? data.at(2) : "" );
		}
        dyn_string exceptionInfo;
		unPanel_setPanelList(dp, labels, names, dollars, exceptionInfo);
        FwException::checkRaise(exceptionInfo);
    }

    public void parseFromMap(shared_ptr<mapping> map){
        _data = map;
    }

    protected void _assertData(){
        FwException::assertNotNull(_data);
    }

    protected void _assertHasKey(const string key){
        FwException::assert(_data.contains(key),"missing key:"+key);
    }

    public string writeToDp(){
        return "";
        //need to initialize it either from read data or from existing one
        //instead of removing need to check if they exist
        // import with cleanup parameter - removes all non-imported nodes
        // repository should hold info if existing or read node
        // only device data is empty after merging and reading structure and hierarhy
        // if _device is empty it means its from existing
    }

};
