/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "classes/fwTree/FwTree.ctl"
#uses "classes/unTree/deviceParsers/unTreeDeviceParser.ctl"

class UnTreeDeviceParserPage : UnTreeDeviceParser {
    public UnTreeDeviceParserPage(
      anytype device,
      shared_ptr<UnTreeProcessContext> context = nullptr) : UnTreeDeviceParser(device,context)
    {
        if(dynlen(_tags) == 0){
            _tags = fwTrending_getPageTreeTagNames();
            for ( int idx = 1 ; idx <= dynlen(_tags) ; ++idx ){
                _tags[idx] = substr(_tags[idx],1);
            }
        }
    }

    
    protected static dyn_string _tags;

    // public string getCustomUserData() {
    //     return "";
    // }

    protected void gatPageData(dyn_dyn_string &ddsPageData, int &itemslen){
        dyn_string exInfo;
        fwTrending_getPage(this.getDeviceDp(), ddsPageData, exInfo);
        FwException::checkRaise(exInfo);

        const int pagelen = dynlen(_tags);
        const int trendlen = dynlen(ddsPageData);
        itemslen = ( trendlen < pagelen ? trendlen : pagelen );
        int lenPS = dynlen(ddsPageData[fwTrending_PAGE_OBJECT_PLOTS]);
        const int lenTP = dynlen(ddsPageData[fwTrending_PAGE_OBJECT_PLOT_TEMPLATE_PARAMETERS]);

        if ( lenPS > 6 ) {
            while ( ddsPageData[fwTrending_PAGE_OBJECT_PLOTS][lenPS] == "" ) {
                dynRemove ( ddsPageData[fwTrending_PAGE_OBJECT_PLOTS] , lenPS );
                if ( --lenPS == 6 ){
                    break;
                }
            }
        }

        if ( lenPS > lenTP ) {
            for ( int i = lenTP ; i < lenPS ; ++i ) {
                dynAppend ( ddsPageData[fwTrending_PAGE_OBJECT_PLOT_TEMPLATE_PARAMETERS] , "" );
            }
        } else if (lenPS < lenTP) {
            for ( int i = lenTP ; i > lenPS ; --i ) {
                dynRemove ( ddsPageData[fwTrending_PAGE_OBJECT_PLOT_TEMPLATE_PARAMETERS] , i );
            }
        }
    }

    public void exportToFile(shared_ptr<UnTreeFileParser> fileParser){
        this.isDeviceOk(UN_TRENDTREE_PAGE); // would throw expection
        _context.addDevice(UnTreeProcessorParam::EXPORTED_DEVICES,this.getDeviceDp());

        dyn_dyn_string ddsPageData;
        int itemslen;
        this.gatPageData(ddsPageData, itemslen);

        anytype currentTop = fileParser.getCurrentTop();
        anytype topNode = fileParser.addNode(UN_TRENDTREE_PAGE);
        fileParser.addValue("dpName",_context.encodeDeviceName(this.getDeviceDp()));

        anytype childNode = fileParser.addNode("pageconfiguration",currentTop);


        for ( int idx = 1 ; idx <= itemslen ; ++idx )
        {
            if ( ! fwTrending_isPageTreeTagExportable(idx) ) {
                continue;
            }

            anytype targetNode;
            if ( dynlen(ddsPageData[idx]) <= 1 ) {
                targetNode = childNode;
            } else {
                targetNode = fileParser.addNode(_tags[idx],childNode);
            }

            for ( int cnt = 1 ; cnt <= dynlen(ddsPageData[idx]) ; ++cnt ) {
                string subDeviceName = ddsPageData[idx][cnt]; 
                if ( idx == fwTrending_PAGE_OBJECT_PLOTS ) {
                    if ( dpExists(subDeviceName) ) {
                        _context.addDevice(UnTreeProcessorParam::MISSING_CANDIDATES,subDeviceName);
                    }
                    subDeviceName = _context.encodeDeviceName(subDeviceName);
                }
                fileParser.addValue(_tags[idx], subDeviceName, targetNode);
            }
        }

        fileParser.setCurrentTop(topNode);
        exportNavigationDevices(fileParser); 
    }
};
