/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

/**
 * This library is dedicated to the panel applicationOperationDetail.pnl,
 * i.e. the function defined in the panel has been moved to this library
 * to ease the editing (nothing else)
 * 
 * 29 July 2021
 * Jean-Charles Tournier
 * CERN 
 * BE-ICS-FT
 * 
 **/

//--------------------------------------------------------------------------------------------------------------------------------
unApplicationOperationDetailPanel_initPanelScript(string applicationName)
{
  int alertType;

  if (dpExists(applicationName) == false)
  {
    DebugTN(__FUNCTION__, "ERROR", "Application name does exist as DP", applicationName);
    return;
  }

  unApplicationOperationDetailPanel_initStatVariable(applicationName);

  applicationDPEList.sortOnClick(TRUE);
  string sSystemName = unGenericDpFunctions_getSystemName(applicationName);
  applicationSystemName.text = sSystemName;

  string tempStr;
  dpGet(applicationName + ".applicationName", tempStr);
  oldApplicationName.text = tempStr;
  contentFrame.text = tempStr;

  dpGet(applicationName + ".defaultPanel", tempStr);
  oldPanelName.text = tempStr;

  dpGet(applicationName + ".applicationName:_alert_hdl.._type", alertType);
  if (alertType != DPCONFIG_SUM_ALERT)
  {
    DebugTN(__FUNCTION__, "ERROR", "No alert handler defined on "+applicationName + ".applicationName:_alert_hdl.._type");
    return;
  }

  int iRes;

  iRes = dpConnect("unApplicationOperationDetailPanel_updateApplicationDPEListCB", 
                    applicationName + ".applicationName:_alert_hdl.._dp_list");
  if (iRes != 0)
  {
    dyn_string exceptionInfo;
    fwException_raise(exceptionInfo, "ERROR", (string)getLastError(), "");
    fwExceptionHandling_display(exceptionInfo);
  }


  iRes = dpConnectUserData("unApplicationOperationDetailPanel_updateApplicationAlertStateCB", sSystemName,
                   applicationName + ".applicationName:_alert_hdl.._act_state_color",
                   applicationName + ".applicationName:_alert_hdl.._active",
                   sSystemName + "_Connections.Ctrl.ManNums");
  if (iRes != 0)
  {
    dyn_string exceptionInfo;
    fwException_raise(exceptionInfo, "ERROR", (string)getLastError(), "");
    fwExceptionHandling_display(exceptionInfo);
  }
}

//--------------------------------------------------------------------------------------------------------------------------------
unApplicationOperationDetailPanel_updateApplicationDPEListCB(string dpe1, dyn_string dsList)
{
  string applicationName, desc;
  string sText;
  int alertType, i, len, iType, ires;

  applicationDPEList.deleteAllLines();
  applicationDPEList.appendLines(dynlen(dsList), "DPE", unApplicationOperationDetailPanel_formatDpeList(dsList));
  g__applicationOperationDetail_dpNames = dsList;
  
  len = dynlen(dsList);
  for (i = 1; i <= len; i++)
  {
    desc = unGenericDpFunctions_getDescription(dsList[i]);
    string alertColour = "unDataNoAccess";
    applicationDPEList.cellBackColRC(i - 1, "AlarmState", alertColour);
    applicationDPEList.cellValueRC(i - 1, "description", desc);
    applicationDPEList.cellValueRC(i - 1, "descriptionDPE", desc);
    applicationDPEList.cellValueRC(i - 1, "AlarmState", "Not Init.");

    dpConnect("unApplicationOperationDetailPanel_alertDPEcb", dsList[i] + ":_alert_hdl.._act_state_color",
              dsList[i] + ":_alert_hdl.._active",
              dsList[i] + ":_online.._text", dsList[i] + ":_online.._stime", dsList[i]);
  }
}

//--------------------------------------------------------------------------------------------------------------------------------
dyn_string unApplicationOperationDetailPanel_formatDpeList(const dyn_string &dsDpeList){
  dyn_string dsFormattedDpeList;
  int iNum=dynlen(dsDpeList);
  for( int i=1; i<=iNum; i++){
    dsFormattedDpeList[i]=dpSubStr( dsDpeList[i], DPSUB_SYS_DP_EL);
  }
  return dsFormattedDpeList;
}

//--------------------------------------------------------------------------------------------------------------------------------
unApplicationOperationDetailPanel_alertDPEcb(string sDpe1, string sAlertColour, string sDpe2, bool bMasked, string sDpe3, string text, string sDpeTime, time tTime, string sDpAlarmValue, int iAlarm)
{
  string tempStr, alertColour, alertText, desc;
  int pos;
  string sTime = tTime;

  alertColour = sAlertColour;

  dyn_string dsDpeInTable = applicationDPEList.getColumnN(applicationDPEList.nameToColumn("DPE"));
  tempStr = dpSubStr(sDpe1, DPSUB_SYS_DP_EL);
  pos = dynContains(dsDpeInTable, tempStr);

  if (pos > 0)
  {
    synchronized(g_oldSelectedRow)
    {
      if (!bMasked)
      {
        alertColour = "unAlarmMasked";
        alertText = "Masked";
      }
      else if (alertColour == "")
      {
        alertColour = "unAlarm_Ok";
        alertText = "OK";
      }
      else
      {
        alertText = "Bad";
      }
      getValue("applicationDPEList", "cellValueRC", pos - 1, "descriptionDPE", desc);
      setValue("applicationDPEList", "cellBackColRC", pos - 1, "AlarmState", alertColour);
      setValue("applicationDPEList", "cellValueRC", pos - 1, "AlarmState", alertText);
      setValue("applicationDPEList", "cellValueRC", pos - 1, "description", sTime + ": " + desc + " " + text);
      setValue("applicationDPEList", "cellValueRC", pos - 1, "alarmValue", iAlarm);
    }
  }
}

//--------------------------------------------------------------------------------------------------------------------------------
unApplicationOperationDetailPanel_initStatVariable( string sApplicationDpName){
  bool bStats = false;
  string sSystemName = dpSubStr(sApplicationDpName, DPSUB_SYS);
  if (dpExists(sSystemName + SYSTEMINTEGRITY_COMPUTESTATISTICS + ".time.timedFunc.interval"))
  {
    dpGet(sSystemName + SYSTEMINTEGRITY_COMPUTESTATISTICS + ".mode", bStats);
  }
  g_bStats = bStats;
}

//--------------------------------------------------------------------------------------------------------------------------------
unApplicationOperationDetailPanel_updateApplicationAlertStateCB(string sSystemName, string dpe2, string alertColour, string dpe3, bool masked, string sDpe3, dyn_int diCtrlMan)
{
  string sText;

  alertColour = "unAlarm_Ok";
  sText = "Running";
  if (!masked)
  {
    alertColour = "unAlarmMasked";
    sText = "Masked";
  }
  else if (alertColour == "")
  {
    alertColour = "unAlarm_Ok";
    sText = "Running";
  }

  if (!unGraphicalFrame_isCtrlScriptRunning(sSystemName, diCtrlMan))
  {
    alertColour = "unAlarm_AnalogHH";
    sText = "Stopped";
  }

  applicationAlertState.backCol = alertColour;
  scriptsStateLabel.text = sText;
}
