/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl" 
#uses "unSystemIntegrity.ctl"
#uses "test/applicationOperation/classes/ApplicationOperationDetailSystemAlarm.ctl"
                                                    

const vector<AppOpSystemAlarm> LIST_SYSTEMALARM = makeVector( AppOpSystemAlarm("AppOpDetail1", "TEST_", "AppOpDetail1"),
                                                              AppOpSystemAlarm("AppOpDetail2", "TEST_", "AppOpDetail2"),
                                                              AppOpSystemAlarm("AppOpDetail3", "TEST_", "AppOpDetail3"),
                                                              AppOpSystemAlarm("AppOpDetail4", "TEST_", "AppOpDetail4"),
                                                              AppOpSystemAlarm("AppOpDetail5", "TEST_", "AppOpDetail5")
                                                            );

const string APP_OPERATION_TEST_PANEL_NAME = "testApplicationOperationDetail";                                                  



//------------------------------------------------------------------
void test_applicationOperationDetail_setupSuite(){
  //Create a couple of system alarm
  int iNum = LIST_SYSTEMALARM.count();
  for( int i=0; i<iNum; i++){
    dyn_string exceptionInfo;
    //create system alarm
    unSystemIntegrity_createSystemAlarm( LIST_SYSTEMALARM[i].getAlarmName(), LIST_SYSTEMALARM[i].getAlarmType(), LIST_SYSTEMALARM[i].getAlarmDescription(), exceptionInfo);
    assertTrue(dynlen(exceptionInfo)==0, "Error while creating alarm: "+exceptionInfo);

    //Add them to the _unApplicationAlarmSummary alert summary
    _unSystemIntegrity_modifyApplicationAlertList(LIST_SYSTEMALARM[i].getDpName(), true, exceptionInfo);
    assertTrue(dynlen(exceptionInfo)==0, "Error while adding alarm to the summary alert handler: "+exceptionInfo);
  }

  //and finally start the panel
  addSymbol( myModuleName(), myPanelName(), "vision/graphicalFrame/applicationOperationDetail.pnl", 
              APP_OPERATION_TEST_PANEL_NAME, makeDynString("$sApplicationDpName:_unApplication"), 1,1, 0, 1,1);
  
  //and wait a bit to make sure the panel is initialized
  delay(5);

}

//------------------------------------
// This routine is optional. It is never called, presence of this routine just tells to
// test runner that tests of this library shall run in UI manager
void test_applicationOperationDetail_alwaysRunInUI()
{
}


//-----------------------------------------------------------------
void test_applicationOperationDetail_checkDisplayedAlarmList(){
  dyn_string dsDisplayedAlarmDpe = invokeMethod(APP_OPERATION_TEST_PANEL_NAME, "getDisplayedAlarmListDpe");
  
  //we except to have all our alarms to be displayed (and some other more but we dont care)
  int iNumDp = LIST_SYSTEMALARM.count();
  for( int i=0; i<iNumDp; i++){
    int iPos = dynContains( dsDisplayedAlarmDpe, LIST_SYSTEMALARM[i].getDpeAlarmName());
    assertTrue( iPos>0, "System Alarm not present in the displayed alarm list: "+LIST_SYSTEMALARM[i].getDpeAlarmName() + " is not in "+dsDisplayedAlarmDpe);
  }
}

//---------------------------------------------------------------------
void test_applicationOperationDetail_checkMaskedAlarm(){
  dpSetWait(LIST_SYSTEMALARM[1].getDpeAlarmName()+":_alert_hdl.._active", "false");
  delay(1); //wait for the CB to be executed in the panel
  
  //Except the color to be yellow (unAlarmMasked)
  string sColorAlarm = invokeMethod(APP_OPERATION_TEST_PANEL_NAME, "getDisplayedAlarmStateColor", LIST_SYSTEMALARM[1].getDpeAlarmName());
  assertEqual("unAlarmMasked", sColorAlarm, "Alarm state color is not the excepted one for "+LIST_SYSTEMALARM[1].getDpeAlarmName());

  //Except the text to be Masked
  string sTextAlarm = invokeMethod(APP_OPERATION_TEST_PANEL_NAME, "getDisplayedAlarmStateText", LIST_SYSTEMALARM[1].getDpeAlarmName());
  assertEqual("Masked", sTextAlarm, "Alarm state text is not the excepted one for "+LIST_SYSTEMALARM[1].getDpeAlarmName());
}

//---------------------------------------------------------------------
void test_applicationOperationDetail_checkActiveAlarmNormalSate(){
  dpSetWait(LIST_SYSTEMALARM[1].getDpeAlarmName()+":_alert_hdl.._active", "true");
  dpSetWait(LIST_SYSTEMALARM[1].getDpeAlarmName(), "0");
  delay(1); //wait for the CB to be executed in the panel
  
  //Except the color to be green (unAlarm_Ok)
  string sColorAlarm = invokeMethod(APP_OPERATION_TEST_PANEL_NAME, "getDisplayedAlarmStateColor", LIST_SYSTEMALARM[1].getDpeAlarmName());
  assertEqual("unAlarm_Ok", sColorAlarm, "Alarm state color is not the excepted one for "+ LIST_SYSTEMALARM[1].getDpeAlarmName());

  //Except the text to be OK
  string sTextAlarm = invokeMethod(APP_OPERATION_TEST_PANEL_NAME, "getDisplayedAlarmStateText", LIST_SYSTEMALARM[1].getDpeAlarmName());
  assertEqual("OK", sTextAlarm, "Alarm state text is not the excepted one for "+LIST_SYSTEMALARM[1].getDpeAlarmName());
}

//---------------------------------------------------------------------
void test_applicationOperationDetail_checkActiveAlarmAlarmSate(){
  int alarm_value = 10;
  dpSetWait(LIST_SYSTEMALARM[2].getDpeAlarmName()+":_alert_hdl.._active", "true");
  dpSetWait(LIST_SYSTEMALARM[2].getDpeAlarmName(), alarm_value);
  delay(1); //wait for the CB to be executed in the panel
  
  //Except the color to be {255,128,0} (defined by _unSystemIntegrityAlarm_lvl1 alert_class)
  string sColorAlarm = invokeMethod(APP_OPERATION_TEST_PANEL_NAME, "getDisplayedAlarmStateColor", LIST_SYSTEMALARM[2].getDpeAlarmName());
  assertEqual("{255,128,0}", sColorAlarm, "Alarm state color is not the excepted one for "+LIST_SYSTEMALARM[2].getDpeAlarmName());

  //Except the text to be running
  string sTextAlarm = invokeMethod(APP_OPERATION_TEST_PANEL_NAME, "getDisplayedAlarmStateText", LIST_SYSTEMALARM[2].getDpeAlarmName());
  assertEqual("Bad", sTextAlarm, "Alarm state text is not the excepted one for "+ LIST_SYSTEMALARM[2].getDpeAlarmName());

  //Except the value to be displayed
  string sValueAlarm = invokeMethod(APP_OPERATION_TEST_PANEL_NAME, "getDisplayedAlarmValue", LIST_SYSTEMALARM[2].getDpeAlarmName());
  assertEqual((string)alarm_value, sValueAlarm, "Alarm value is not the excepted one for "+ LIST_SYSTEMALARM[2].getDpeAlarmName());
}