/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/

#uses "fwUnitTestComponentAsserts"
#uses "AlarmScreenNg/classes/AsNgActions"
#uses "AlarmScreenNg/classes/AsNgPopupMenu"
#uses "AlarmScreenNg/classes/AsNgFilterViewActions"

private const string AS_EWO_TEST_PANEL = "vision/test/fwAlarmScreenNg.pnl";

private const string AS_EWO_CONTROL_NAME = "AsEwo";

private const string AS_EWO_SYMBOL_NAME = "SymbAsEwo";

private const string TEST_ALARM_DPE_TYPE = "_NgAsTestAlarm";

private const string TEST_USER_SETTING_DP = "_NgAsUserSettingTest";

private const string TEST_FILTER_DP = "_NgAsFilterTest";

private const string TEST_ADMIN_DP = "_NgAsAdminTest";

private const string TEST_FILTER_SET_DP = "_NgAsFilterSetTest";

private const string TEST_DPE_BASENAME = "TestAlarm_Group_";

private const int NO_OF_ALARMS = 10;

// Override load method to load data from Test DPs.
class TestAsNgActions : AsNgActions {
  public bool load(dyn_string &dsExceptions) {
    clear();
    string sJson;

    dpGet(TEST_ADMIN_DP + "." + STORAGE_DPE_NAME, sJson);
    if(sJson.isEmpty()) {
      return true;  // empty string is empty, but still valid, configuration
    }
    mapping dmItems = jsonDecode(sJson);
    dyn_errClass deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to parse JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    return populateItems(dmItems["actions"], dsExceptions);
  }
};

shared_ptr<TestAsNgActions> actions;

class TestPopupMenu : AsNgPopupMenu {
  public TestPopupMenu(shared_ptr<AsNgActions> actions) : AsNgPopupMenu(actions){
  }

  public bool load(dyn_string &dsExceptions) {
    clear();

    string sJson;
    dpGet(TEST_ADMIN_DP + "." + STORAGE_DPE_NAME, sJson);

    if(sJson.isEmpty()) {
      return true;  // empty string is empty, but still valid, configuration
    }
    mapping dmItems = jsonDecode(sJson);
    dyn_errClass deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to parse JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    return populateItems(dmItems["items"], dsExceptions);
  }
};

class TestFilterViewActions: AsNgFilterViewPopupMenu {
  public TestFilterViewActions(shared_ptr<AsNgActions> actions) : AsNgFilterViewPopupMenu(actions){
  }

  public bool load(dyn_string &dsExceptions) {
    clear();

    string sJson;
    dpGet(TEST_ADMIN_DP + "." + STORAGE_DPE_NAME, sJson);

    if(sJson.isEmpty()) {
      return true;  // empty string is empty, but still valid, configuration
    }
    mapping dmItems = jsonDecode(sJson);
    dyn_errClass deErrors = getLastError();
    if(deErrors.count() > 0) {
      fwException_raise(dsExceptions, "ERROR", __FUNCTION__ +
                        "(): failed to parse JSON string from DPE: " + getErrorText(deErrors), "");
      throwError(deErrors);
      return false;
    }

    return populateItems(dmItems["items"], dsExceptions);
  }
};

void AsNgActions_test_addTestPanel(){
    addSymbol(myModuleName(), myPanelName(), AS_EWO_TEST_PANEL, AS_EWO_SYMBOL_NAME, makeDynString());
}

void AsNgActions_test_removeTestPanel(){
    if(!shapeExists(AS_EWO_SYMBOL_NAME))
    {
      return;
    }
    //DebugN(__FUNCTION__ + "(): removing symbol", sSymbolName);
    removeSymbol(myModuleName(), myPanelName(), AS_EWO_SYMBOL_NAME);
}

shape AsNgActions_test_getEwoShape()
{
  string sShapeName = AS_EWO_SYMBOL_NAME + "." + AS_EWO_CONTROL_NAME;
  shape ewo = getShape(sShapeName);
  if(ewo == 0)
  {
    DebugN("shape not found", sShapeName);
  }
  ewo.ownerPanelReady();
  ewo.enableWarningPopup(false);
  return ewo;
}


void AsNgActions_test_createAlarms(int nbOfAlarms, dyn_string &dsExceptions){
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + ((i < nbOfAlarms/2) ? "A_" : "B_") + i;
    if(!dpExists(dpName)){
      dpCreate(dpName, TEST_ALARM_DPE_TYPE);
      dyn_errClass deErrors = getLastError();
      if(dynlen(deErrors) > 0){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ + "(): failed to create DP '" +
                        dpName + "': " + getErrorText(deErrors), "");
        throwError(deErrors);
        return;
      }
      while(!dpExists(dpName)){
        delay(0, 100);
      }
    }

    dyn_mixed alarmObject;

    fwAlertConfig_objectCreateAnalog(
        alarmObject,
        makeDynString("COLD", "NORMAL", "HOT"),
        makeDynFloat(0, 22, 33),
        makeDynString("_fwWarningAck.", "", "_fwErrorAck."),
        "",
        makeDynString(""),
        "",
        makeDynBool(0, 1, 0),
        dsExceptions);
//"dist_1:TestAlarm_0.ProcessInput.dpeInt""dist_1:TestAlarm_0.ProcessInput"
    fwAlertConfig_objectSet(dpName + ".IntVal", alarmObject, dsExceptions);
    if(dynlen(dsExceptions) > 0){
      DebugN(dsExceptions);
    }
    fwAlertConfig_activate(dpName + ".IntVal", dsExceptions);
  }
}


void AsNgActions_test_removeAlarms(int nbOfAlarms, dyn_string &dsExceptions){
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + i;
    dpName += (i < nbOfAlarms/2) ? "_GROUP_A" : "_GROUP_B";
    if(dpExists(dpName)){
      dpDelete(dpName, TEST_ALARM_DPE_TYPE);
      dyn_errClass deErrors = getLastError();
      if(dynlen(deErrors) > 0){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ + "(): failed to create DP '" +
                        dpName + "': " + getErrorText(deErrors), "");
        throwError(deErrors);
        return;
      }
    }
  }
}

void test_AsNgActions_setupSuite(){
  // Do nothing
}
void test_AsNgActions_setup(){
  AsNgActions_test_addTestPanel();
  actions = new TestAsNgActions();
  dyn_string dsExceptions;
  actions.load(dsExceptions);
  dyn_string dsExceptions;
  AsNgActions_test_createAlarms(NO_OF_ALARMS, dsExceptions);
}
void test_AsNgActions_teardown(){
  AsNgActions_test_removeTestPanel();
  dyn_string dsExceptions;
  AsNgActions_test_removeAlarms(NO_OF_ALARMS, dsExceptions);
}
void test_AsNgActions_teardownSuite(){
  // Do nothing
}

void test_AsNgActions_AsNgActions_alwaysRunInUI(){
  // routine to run tests in UI manager.
}

void AsNgActions_test_assertCount(int expectedCount, int actualCount){
  assert(actualCount == expectedCount, "Expected Count: " + expectedCount + "\nActual Count: " + actualCount);
}

// Check if valid alarm actions are loaded.
void test_AsNgActions_validAlarmActions(){
  dyn_string actualNames = actions.getActionsByParam("ALARM");
  for(int i = 1; i<=7; i++){
    assert("AlarmScreenAction"+i == actualNames.at(i-1),
    "Invalid Alarm action loaded:\nExpected: " + "AlarmScreenAction"+i + "\nActual: " + actualNames.at(i-1));
  }
}

// Check if valid filter view actions are loaded.
void test_AsNgActions_validFilterActions(){
  dyn_string actualNames = actions.getActionsByParam("FILTER");
  for(int i = 1; i<=7; i++){
    assert("FilterViewAction"+i == actualNames.at(i-1),
    "Invalid Alarm action loaded:\nExpected: " + "AlarmScreenAction"+i + "\nActual: " + actualNames.at(i-1));
  }
}

// Check if a particular action exists.
void test_AsNgActions_actionNameExistence(){
  string expectedAction = "FilterViewAction1";
  assert(actions.actionNameExists(expectedAction),
         expectedAction + " not found!");

  string unexpectedAction = "FilterViewAction8";
  assert(!actions.actionNameExists(unexpectedAction),
         unexpectedAction + " should not exist!");
}

// test successful ewo creation
void test_AsNgActions_EwoCreation() {
  shape ewo = AsNgActions_test_getEwoShape();
  delay(10);
  AsNgActions_test_assertCount(NO_OF_ALARMS, dynlen(ewo.getAllRowsData()));
}

void test_AsNgActions_applyFilter(){
  shape ewo = AsNgActions_test_getEwoShape();

  string sJson;
  dpGet(TEST_FILTER_DP + ".FilterJSON", sJson);

  ewo.applyFilter(sJson);
  delay(10);
  AsNgActions_test_assertCount(NO_OF_ALARMS/2, dynlen(ewo.getAllRowsData()));
}

void test_AsNgActions_PopupMenuItems(){
  dyn_string dsExceptions;
  shared_ptr<TestPopupMenu> menu = new TestPopupMenu(actions);
  menu.load(dsExceptions);

  AsNgActions_test_assertCount(6, menu.itemCount());

  vector<shared_ptr<AsNgPopupMenuItem> > items = menu.getItems();
  string base_name = "PopupMenuItem";
  for(int i=1; i<=items.count(); i++){
    assert(base_name+i == items.at(i-1).getLabel(), "Expected: " + base_name+i + "\nActual :", items.at(i-1).getLabel());
  }
}

void test_AsNgActions_FilterViewItems(){
  dyn_string dsExceptions;
  shared_ptr<TestFilterViewActions> menu = new TestFilterViewActions(actions);
  menu.load(dsExceptions);

  AsNgActions_test_assertCount(6, menu.itemCount());
  assert("FilterViewAction7", menu.getLeftClickActionName());
  vector<shared_ptr<AsNgPopupMenuItem> > items = menu.getItems();
  string base_name = "FilterViewItem";
  for(int i=1; i<=items.count(); i++){
    assert(base_name+i == items.at(i-1).getLabel(), "Expected: " + base_name+i + "\nActual :", items.at(i-1).getLabel());
  }
}

// Test if actions are triggered correctly.
void test_AsNgActions_actionProcessing(){
  dyn_string dsExceptions;
  shared_ptr<TestFilterViewActions> menu = new TestFilterViewActions(actions);
  menu.load(dsExceptions);

  string sJson;
  dpGet(TEST_FILTER_DP + ".FilterJSON", sJson);

  mapping mArgs;
  shape ewo = AsNgActions_test_getEwoShape();
  mArgs["ewoShape"] = ewo;
  mArgs["filterJson"] = sJson;
  menu.getItem(0).process(mArgs, dsExceptions);
  delay(10);
  AsNgActions_test_assertCount(NO_OF_ALARMS/2, dynlen(ewo.getAllRowsData()));
}
