/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/

#uses "fwUnitTestComponentAsserts"
#uses "AlarmScreenNg/classes/AsNgActions"
#uses "AlarmScreenNg/classes/AsNgPopupMenu"
#uses "AlarmScreenNg/classes/AsNgFilterViewActions"
#uses "AlarmScreenNg/AlarmScreenNgFilters"

private const string AS_EWO_TEST_PANEL = "vision/test/fwAlarmScreenNg.pnl";

private const string AS_EWO_CONTROL_NAME = "AsEwo";

private const string AS_EWO_SYMBOL_NAME = "SymbAsEwo";

private const string TEST_ALARM_DPE_TYPE = "_NgAsTestAlarm";

private const string TEST_USER_SETTING_DP = "_NgAsUserSettingTest";

private const string TEST_FILTER_DP = "_NgAsFilterTest";

private const string TEST_ADMIN_DP = "_NgAsAdminTest";

private const string TEST_FILTER_SET_DP = "_NgAsFilterSetTest";

private const string TEST_DPE_BASENAME = "TestAlarm_Group_";

private const string TEST_FILTER_NAME = "Test Filter";

private const string TEST_FILTER_DP_SAVE = "_NgAsFilterTestSave";

private const int NO_OF_ALARMS = 10;

void test_AsNgFooterFilterStatistics_setupSuite(){
  // Do nothing
}
void test_AsNgFooterFilterStatistics_setup(){
  AsNgFooterFilterStatistics_test_addTestPanel();
  dyn_string dsExceptions;
  createAlarms(NO_OF_ALARMS, dsExceptions);
}
void test_AsNgFooterFilterStatistics_teardown(){
  AsNgFooterFilterStatistics_test_removeTestPanel();
  dyn_string dsExceptions;
  AsNgFooterFilterStatistics_test_removeAlarms(NO_OF_ALARMS, dsExceptions);
}
void test_AsNgFooterFilterStatistics_teardownSuite(){
  // Do nothing
}

void test_AsNgFooterFilterStatistics_AsNgFooterFilterStatistics_alwaysRunInUI(){
  // routine to run tests in UI manager.
}

void test_AsNgFooterFilterStatistics_setEmptyFooter(){
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("");
  delay(10);
  string footerText = ewo.getFooterText();
  assert(footerText.isEmpty(), __FUNCTION__ + "(): Footer text must be empty instead of: " + footerText);
}

void test_AsNgFooterFilterStatistics_setDefaultFilter()
{
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("Active Filter: {{filter_name}}");
  //  ewo.applyFilter(?, ?); // how to apply empty filter?
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgFooterFilterStatistics_test_assertString("Active Filter: No Filter", footerText);
  // "internalCounter" always set in AlarmsProxyModel::AlarmsProxyModel
}

void test_AsNgFooterFilterStatistics_setCustomFilter()
{
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("Active Filter: {{filter_name}}");
  string sJson, filterName;
  dpGet(TEST_FILTER_DP + ".FilterName", filterName);
  dpGet(TEST_FILTER_DP + ".FilterJSON", sJson);

  ewo.applyFilter(sJson, filterName);
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgFooterFilterStatistics_test_assertString("Active Filter: Filter_GroupA", footerText);
}

void test_AsNgFooterFilterStatistics_setTestFilter()
{
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("Active Filter: {{filter_name}}");

  string sJson, sNewJson, filterName, newFilterName;
  dpGet(TEST_FILTER_DP + ".FilterName", filterName);
  dpGet(TEST_FILTER_DP + ".FilterJSON", sJson);

  dyn_string exceptionInfo;
  AlarmScreenNgSaveResult result = AlarmScreenNgFilters_saveFilter(TEST_FILTER_DP_SAVE, sJson, "test", exceptionInfo, true);

  dpGet(TEST_FILTER_DP + ".FilterJSON", sNewJson);
  dpGet(TEST_FILTER_DP + ".FilterName", newFilterName);

  ewo.applyFilter(sNewJson, newFilterName);
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgFooterFilterStatistics_test_assertString("Active Filter: " + newFilterName, footerText);
}

void test_AsNgFooterFilterStatistics_setTotalAlarms()
{
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("Total Alarms: {{total_alarms_count}}");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgFooterFilterStatistics_test_assertString("Total Alarms: " + NO_OF_ALARMS, footerText);
}

void test_AsNgFooterFilterStatistics_setFilteredAlarms_noFilteredAlarms()
{
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("Filtered Alarms: {{filtered_alarms_count}}");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgFooterFilterStatistics_test_assertString("Filtered Alarms: " + NO_OF_ALARMS, footerText);
}

void test_AsNgFooterFilterStatistics_setFilteredAlarms_appliedFilter()
{
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("Filtered Alarms: {{filtered_alarms_count}}");
  delay(10);

  string sJson, filterName;
  dpGet(TEST_FILTER_DP + ".FilterName", filterName);
  dpGet(TEST_FILTER_DP + ".FilterJSON", sJson);
  ewo.applyFilter(sJson, filterName);

  delay(10);
  string footerText = ewo.getFooterText();
  AsNgFooterFilterStatistics_test_assertString("Filtered Alarms: " + NO_OF_ALARMS/2, footerText);
}

void test_AsNgFooterFilterStatistics_setUnackedAlarms_allUnack()
{
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("Unack: {{unack_total_count}}");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgFooterFilterStatistics_test_assertString("Unack: " + NO_OF_ALARMS, footerText);
}

void test_AsNgFooterFilterStatistics_setUnackedAlarms_allAck()
{
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  ewo.setFooterText("Unack: {{unack_total_count}}");
  delay(10); // not sure why we need delay this long
  AsNgFooterFilterStatistics_test_acknowledgeAlarms(NO_OF_ALARMS);
  delay(10); // not sure why we need delay this long
  string footerText = ewo.getFooterText();
  AsNgFooterFilterStatistics_test_assertString("Unack: 0", footerText);
}

void AsNgFooterFilterStatistics_test_addTestPanel() {
  addSymbol(myModuleName(), myPanelName(), AS_EWO_TEST_PANEL, AS_EWO_SYMBOL_NAME, makeDynString());
}

void AsNgFooterFilterStatistics_test_removeTestPanel(){
  if(!shapeExists(AS_EWO_SYMBOL_NAME)) {
    removeSymbol(myModuleName(), myPanelName(), AS_EWO_SYMBOL_NAME);
  }
}

shape AsNgFooterFilterStatistics_test_getEwoShape()
{
  string sShapeName = AS_EWO_SYMBOL_NAME + "." + AS_EWO_CONTROL_NAME;
  shape ewo = getShape(sShapeName);
  if(ewo == 0)
  {
    DebugN("shape not found", sShapeName);
  }
  ewo.ownerPanelReady();
  return ewo;
}


void createAlarms(int nbOfAlarms, dyn_string &dsExceptions){
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + ((i < nbOfAlarms/2) ? "A_" : "B_") + i;
    if(!dpExists(dpName)){
      dpCreate(dpName, TEST_ALARM_DPE_TYPE);
      dyn_errClass deErrors = getLastError();
      if(dynlen(deErrors) > 0){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ + "(): failed to create DP '" +
                        dpName + "': " + getErrorText(deErrors), "");
        throwError(deErrors);
        return;
      }
      while(!dpExists(dpName)){
        delay(0, 100);
      }
    }

    dyn_mixed alarmObject;

    fwAlertConfig_objectCreateAnalog(
        alarmObject,
        makeDynString("COLD", "NORMAL", "HOT"),
        makeDynFloat(0, 22, 33),
        makeDynString("_fwWarningAck.", "", "_fwErrorAck."),
        "",
        makeDynString(""),
        "",
        makeDynBool(0, 1, 0),
        dsExceptions);
//"dist_1:TestAlarm_0.ProcessInput.dpeInt""dist_1:TestAlarm_0.ProcessInput"
    fwAlertConfig_objectSet(dpName + ".IntVal", alarmObject, dsExceptions);
    if(dynlen(dsExceptions) > 0){
      DebugN(dsExceptions);
    }
    fwAlertConfig_activate(dpName + ".IntVal", dsExceptions);
  }
}


// TODO: this AsNgFooterFilterStatistics_test_removeAlarms does not work (wrong alarm names)
// when removing the proper ones, tests interfere and fail!
void AsNgFooterFilterStatistics_test_removeAlarms(int nbOfAlarms, dyn_string &dsExceptions){
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + i;
    dpName += (i < nbOfAlarms/2) ? "_GROUP_A" : "_GROUP_B";
    if(dpExists(dpName)){
      dpDelete(dpName, TEST_ALARM_DPE_TYPE);
      dyn_errClass deErrors = getLastError();
      if(dynlen(deErrors) > 0){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ + "(): failed to create DP '" +
                        dpName + "': " + getErrorText(deErrors), "");
        throwError(deErrors);
        return;
      }
    }
  }
}

void AsNgFooterFilterStatistics_test_assertCount(int expectedCount, int actualCount){
  assert(actualCount == expectedCount, "Expected Count: " + expectedCount + "\nActual Count: " + actualCount);
}

void AsNgFooterFilterStatistics_test_assertString(string expectedString, string actualString){
  assert(actualString == expectedString, "Expected String: " + expectedString + "\nActual String: " + actualString);

}

// test successful ewo creation
void test_AsNgFooterFilterStatistics_EwoCreation() {
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();
  delay(10);
  AsNgFooterFilterStatistics_test_assertCount(NO_OF_ALARMS, dynlen(ewo.getAllRowsData()));
}

void test_AsNgFooterFilterStatistics_applyFilter(){
  shape ewo = AsNgFooterFilterStatistics_test_getEwoShape();

  string sJson;
  dpGet(TEST_FILTER_DP + ".FilterJSON", sJson);

  ewo.applyFilter(sJson, "test_name");
  AsNgFooterFilterStatistics_test_assertCount(NO_OF_ALARMS/2, dynlen(ewo.getAllRowsData()));
}

// ack first nbOfAlarms alarms
void AsNgFooterFilterStatistics_test_acknowledgeAlarms(const int nbOfAlarms) {
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + ((i < NO_OF_ALARMS/2) ? "A_" : "B_") + i;
    if(dpExists(dpName)) {
        dpSetWait(dpName + ".IntVal:_alert_hdl.._ack", true);
    }
  }
}
