/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/

#uses "fwUnitTestComponentAsserts"

private const string AS_EWO_TEST_PANEL = "vision/test/fwAlarmScreenNg.pnl";

private const string AS_EWO_CONTROL_NAME = "AsEwo";

private const string AS_EWO_SYMBOL_NAME = "SymbAsEwo";

private const string TEST_ALARM_DPE_TYPE = "_NgAsTestAlarm";

private const string TEST_USER_SETTING_DP = "_NgAsUserSettingTest";

private const string TEST_FILTER_DP = "_NgAsFilterTest";

private const string TEST_ADMIN_DP = "_NgAsAdminTest";

private const string TEST_DPE_BASENAME = "TestAlarm_Group_";

private const int NO_OF_ALARMS_BEFORE_PAUSE = 3;

private const int NO_OF_ALARMS_AFTER_PAUSE = 2;

private const string ALARM_SUFFIX_BEFORE_PAUSE = "BEFORE_PAUSE";

private const string ALARM_SUFFIX_AFTER_PAUSE = "AFTER_PAUSE";

void AsNgPauseMode_addTestPanel(){
  addSymbol(myModuleName(), myPanelName(), AS_EWO_TEST_PANEL, AS_EWO_SYMBOL_NAME, makeDynString());
}

void AsNgPauseMode_removeTestPanel(){
  if(!shapeExists(AS_EWO_SYMBOL_NAME)) {
    return;
  }
  //DebugN(__FUNCTION__ + "(): removing symbol", sSymbolName);
  removeSymbol(myModuleName(), myPanelName(), AS_EWO_SYMBOL_NAME);
}

shape AsNgPauseMode_test_getEwoShape()
{
  string sShapeName = AS_EWO_SYMBOL_NAME + "." + AS_EWO_CONTROL_NAME;
  shape ewo = getShape(sShapeName);
  if(ewo == 0)
  {
    DebugN("shape not found", sShapeName);
  }
  ewo.ownerPanelReady();
  ewo.enableWarningPopup(false);
  return ewo;
}

void AsNgPauseMode_test_createAlarms(int nbOfAlarms, string suffix, dyn_string &dsExceptions){
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + suffix + i;
    if(!dpExists(dpName)){
      dpCreate(dpName, TEST_ALARM_DPE_TYPE);
      dyn_errClass deErrors = getLastError();
      if(dynlen(deErrors) > 0){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ + "(): failed to create DP '" +
                        dpName + "': " + getErrorText(deErrors), "");
        throwError(deErrors);
        return;
      }
      while(!dpExists(dpName)){
        delay(0, 100);
      }
    }

    dyn_mixed alarmObject;

    fwAlertConfig_objectCreateAnalog(
        alarmObject,
        makeDynString("COLD", "HOT"),
        makeDynFloat(0, 9),
        makeDynString("_fwErrorAck.", ""),
        "",
        makeDynString(""),
        "",
        makeDynBool(0, 1),
        dsExceptions);
//"dist_1:TestAlarm_0.ProcessInput.dpeInt""dist_1:TestAlarm_0.ProcessInput"
    fwAlertConfig_objectSet(dpName + ".IntVal", alarmObject, dsExceptions);
    if(dynlen(dsExceptions) > 0){
      DebugN(dsExceptions);
    }
    fwAlertConfig_activate(dpName + ".IntVal", dsExceptions);
  }
}

void AsNgPauseMode_test_removeAlarms(int nbOfAlarms, string suffix, dyn_string &dsExceptions){
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + suffix + i;
    if(dpExists(dpName)){
      dpDelete(dpName, TEST_ALARM_DPE_TYPE);
      dyn_errClass deErrors = getLastError();
      if(dynlen(deErrors) > 0){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ + "(): failed to create DP '" +
                        dpName + "': " + getErrorText(deErrors), "");
        throwError(deErrors);
        return;
      }
    }
  }
}

void test_AsNgPauseMode_setupSuite(){
  // Do nothing
}

void test_AsNgPauseMode_setup(){
  AsNgPauseMode_addTestPanel();
  dyn_string dsExceptions;
  AsNgPauseMode_test_createAlarms(NO_OF_ALARMS_BEFORE_PAUSE, ALARM_SUFFIX_BEFORE_PAUSE, dsExceptions);
}

void test_AsNgPauseMode_teardown() {
  AsNgPauseMode_removeTestPanel();
  dyn_string dsExceptions;
  AsNgPauseMode_test_removeAlarms(NO_OF_ALARMS_BEFORE_PAUSE, ALARM_SUFFIX_BEFORE_PAUSE, dsExceptions);
  AsNgPauseMode_test_removeAlarms(NO_OF_ALARMS_AFTER_PAUSE, ALARM_SUFFIX_AFTER_PAUSE, dsExceptions);
}

void test_AsNgPauseMode_teardownSuite(){
  // Do nothing
}

void test_AsNgPauseMode_alwaysRunInUI(){
  // routine to run tests in UI manager
}

void assertCount(int expectedCount, int actualCount){
  assert(actualCount == expectedCount, "Expected Count: " + expectedCount + "\nActual Count: " + actualCount);
}

void test_AsNgPauseMode_EwoCreation() {
  shape ewo = AsNgPauseMode_test_getEwoShape();
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE, dynlen(ewo.getAllRowsData()));
}

void test_AsNgPauseMode_PauseModeEnablement(){
  shape ewo = AsNgPauseMode_test_getEwoShape();
  assert(ewo.test_isPauseModeEnabled(), "Paused Mode was expected to be enabled.");

  ewo.basicConfigDp = "_NgAsConfig_JCOP";
  assert(!ewo.test_isPauseModeEnabled());
}

void test_AsNgPauseMode_countAlarmsBeforeAndAfterPause(){
  shape ewo = AsNgPauseMode_test_getEwoShape();
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE, dynlen(ewo.getAllRowsData()));

  ewo.test_pauseAlarmScreen();
  dyn_string dsExceptions;
  AsNgPauseMode_test_createAlarms(NO_OF_ALARMS_AFTER_PAUSE, ALARM_SUFFIX_AFTER_PAUSE, dsExceptions);

  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE, dynlen(ewo.getAllRowsData()));

  ewo.test_resumeAlarmScreen();
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE + NO_OF_ALARMS_AFTER_PAUSE, dynlen(ewo.getAllRowsData()));
}

void test_AsNgPauseMode_autoResume() {
  shape ewo = AsNgPauseMode_test_getEwoShape();
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE, dynlen(ewo.getAllRowsData()));

  ewo.test_pauseAlarmScreen();
  dyn_string dsExceptions;
  AsNgPauseMode_test_createAlarms(NO_OF_ALARMS_AFTER_PAUSE, ALARM_SUFFIX_AFTER_PAUSE, dsExceptions);

  delay(35); // Pause mode supposed to resume after 30 sec
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE + NO_OF_ALARMS_AFTER_PAUSE, dynlen(ewo.getAllRowsData()));
}

void test_AsNgPauseMode_renewedAlarm(){
  shape ewo = AsNgPauseMode_test_getEwoShape();
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE, dynlen(ewo.getAllRowsData()));

  ewo.test_pauseAlarmScreen();

  dyn_anytype allRows = ewo.getAllRowsData();
  int randomAlarmIdx = (rand() % NO_OF_ALARMS_BEFORE_PAUSE) + 1;
  string alarmDpName = allRows[randomAlarmIdx]["DataPointName"];
  assert(allRows[randomAlarmIdx]["Direction"]);

  dpSetWait(alarmDpName + ".IntVal", 10);
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE+1, dynlen(ewo.getAllRowsData()));

  dpSetWait(alarmDpName + ".IntVal", 0);
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE-1, dynlen(ewo.getAllRowsData()));

  ewo.test_resumeAlarmScreen();
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE, dynlen(ewo.getAllRowsData()));
}

void test_AsNgPauseMode_ackedAlarm(){
  shape ewo = AsNgPauseMode_test_getEwoShape();
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE, dynlen(ewo.getAllRowsData()));

  ewo.test_pauseAlarmScreen();

  dyn_anytype allRows = ewo.getAllRowsData();
  int randomAlarmIdx = (rand() % NO_OF_ALARMS_BEFORE_PAUSE) + 1;
  string alarmDpName = allRows[randomAlarmIdx]["DataPointName"];
  assert(allRows[randomAlarmIdx]["Direction"]);

  dpSetWait(alarmDpName + ".IntVal", 10);
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE+1, dynlen(ewo.getAllRowsData()));

  AsNgPauseMode_test_acknowledgeAlarm(alarmDpName);
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE-1, dynlen(ewo.getAllRowsData()));

  dyn_string dsExceptions;
  AsNgPauseMode_test_createAlarms(NO_OF_ALARMS_AFTER_PAUSE, ALARM_SUFFIX_AFTER_PAUSE, dsExceptions);
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE-1, dynlen(ewo.getAllRowsData()));

  ewo.test_resumeAlarmScreen();
  delay(5);
  assertCount(NO_OF_ALARMS_BEFORE_PAUSE + NO_OF_ALARMS_AFTER_PAUSE - 1 , dynlen(ewo.getAllRowsData()));
}

void AsNgPauseMode_test_acknowledgeAlarm(string alarmDpName) {
  if(dpExists(alarmDpName)) {
    dpSetWait(alarmDpName + ".IntVal:_alert_hdl.._ack", true);
  }
}
