/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
  icecontrols.support@cern.ch

  SPDX-License-Identifier: Apache-2.0
**/

#uses "fwUnitTestComponentAsserts"
#uses "AlarmScreenNg/classes/AsNgActions"
#uses "AlarmScreenNg/classes/AsNgPopupMenu"
#uses "AlarmScreenNg/classes/AsNgFilterViewActions"

private const string AS_EWO_TEST_PANEL = "vision/test/fwAlarmScreenNg.pnl";

private const string AS_EWO_CONTROL_NAME = "AsEwo";

private const string AS_EWO_SYMBOL_NAME = "SymbAsEwo";

private const string TEST_ALARM_DPE_TYPE = "_NgAsTestAlarm";

private const string TEST_USER_SETTING_DP = "_NgAsUserSettingTest";

private const string TEST_FILTER_DP = "_NgAsFilterTest";

private const string TEST_ADMIN_DP = "_NgAsAdminTest";

private const string TEST_FILTER_SET_DP = "_NgAsFilterSetTest";

private const string TEST_DPE_BASENAME = "TestAlarm_Group_";

private const string TEST_FILTER_NAME = "Test Filter";

private const string TEST_FILTER_DP_SAVE = "_NgAsFilterTestSave";

private const int NO_OF_ALARMS = 10;

void test_AsNgRegexParser_setupSuite(){
  // Do nothing
}
void test_AsNgRegexParser_setup(){
  AsNgRegexParser_test_addTestPanel();
  dyn_string dsExceptions;
  dyn_string dsExceptions;
  AsNgRegexParser_test_createAlarms(NO_OF_ALARMS, dsExceptions);
}
void test_AsNgRegexParser_teardown(){
  AsNgRegexParser_test_removeTestPanel();
  dyn_string dsExceptions;
  AsNgRegexParser_test_removeAlarms(NO_OF_ALARMS, dsExceptions);
}
void test_AsNgRegexParser_teardownSuite(){
  // Do nothing
}

void test_AsNgRegexParser_AsNgRegexParser_alwaysRunInUI(){
  // routine to run tests in UI manager.
}

void test_AsNgRegexParser_setEmptyString(){
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("");
  delay(10);
  string footerText = ewo.getFooterText();
  assert(footerText.isEmpty(), __FUNCTION__ + "(): Footer text must be empty instead of: " + footerText);
}

void test_AsNgRegexParser_setConstantString()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  const string parseConst = "Const string, no formating";
  delay(10);
  ewo.setFooterText(parseConst);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString(parseConst, footerText);
}

void test_AsNgRegexParser_setSingleParameter()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("{{total_alarms_count}}");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString(NO_OF_ALARMS, footerText);
}

void test_AsNgRegexParser_setAllParameters()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  const string fullyParameterized = "{{filter_name}} " +
				    "{{total_alarms_count}} " +
				    "{{unack_total_count}}";
  // no filters applied - statistics are equivalent to total number
  const string expectedResult = "No Filter " +
                                NO_OF_ALARMS + " " +
			        NO_OF_ALARMS;
  ewo.setFooterText(fullyParameterized);
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString(expectedResult, footerText);
}

void test_AsNgRegexParser_setHexString()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("\x68\x65\x78\x65\x64");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString("hexed", footerText);
}

void test_AsNgRegexParser_setStringBeforeParameter()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("Total Alarms: {{total_alarms_count}}");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString("Total Alarms: " + NO_OF_ALARMS, footerText);
}

void test_AsNgRegexParser_setStringAfterParameter()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("{{total_alarms_count}} (Total Alarms)");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString(NO_OF_ALARMS + " (Total Alarms)", footerText);
}

void test_AsNgRegexParser_setStringBetweenParameters()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("{{filtered_alarms_count}} / {{total_alarms_count}}");
  delay(10);
  string footerText = ewo.getFooterText();
  // no filter is being applied so filtered_alarms_count will display same as total
  AsNgRegexParser_test_assertString(NO_OF_ALARMS + " / " + NO_OF_ALARMS, footerText);
}

void test_AsNgRegexParser_setUnclosedLeftBrackets()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("{{");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString("{{", footerText);
}

void test_AsNgRegexParser_setUnclosedLeftBracketsBetweenString()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  const string bracketsBetweenString = "total alarms {{total_alarms";
  ewo.setFooterText(bracketsBetweenString);
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString(bracketsBetweenString, footerText);
}
void test_AsNgRegexParser_setUnclosedRightBrackets()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("}}");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString("}}", footerText);
}

void test_AsNgRegexParser_setUnclosedRightBracketsBetweenString()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  const string bracketsBetweenString = "total_alarms}} (total alarms)";
  ewo.setFooterText(bracketsBetweenString);
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString(bracketsBetweenString, footerText);
}

void test_AsNgRegexParser_setUnknownKeyword()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("{{invalid_count}}");
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString("invalid-parameter", footerText);
}

void test_AsNgRegexParser_setEmptyKeyword()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  const string emptyBrackets = "{{}}";
  ewo.setFooterText(emptyBrackets);
  delay(10);
  string footerText = ewo.getFooterText();
  AsNgRegexParser_test_assertString(emptyBrackets, footerText);
}

void test_AsNgRegexParser_setContiguousKeywords()
{
  shape ewo = AsNgRegexParser_test_getEwoShape();
  ewo.setFooterText("{{filtered_alarms_count}}{{total_alarms_count}}");
  delay(10);
  string footerText = ewo.getFooterText();
  // no filter is being applied so filtered_alarms_count will display same as total
  AsNgRegexParser_test_assertString(NO_OF_ALARMS + "" + NO_OF_ALARMS, footerText);
}


void AsNgRegexParser_test_addTestPanel(){
    addSymbol(myModuleName(), myPanelName(), AS_EWO_TEST_PANEL, AS_EWO_SYMBOL_NAME, makeDynString());
}

void AsNgRegexParser_test_removeTestPanel(){
    if(!shapeExists(AS_EWO_SYMBOL_NAME))
    {
      return;
    }
    //DebugN(__FUNCTION__ + "(): removing symbol", sSymbolName);
    removeSymbol(myModuleName(), myPanelName(), AS_EWO_SYMBOL_NAME);
}

shape AsNgRegexParser_test_getEwoShape()
{
  string sShapeName = AS_EWO_SYMBOL_NAME + "." + AS_EWO_CONTROL_NAME;
  shape ewo = getShape(sShapeName);
  if(ewo == 0)
  {
    DebugN("shape not found", sShapeName);
  }
  ewo.ownerPanelReady();
  return ewo;
}


void AsNgRegexParser_test_createAlarms(int nbOfAlarms, dyn_string &dsExceptions){
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + ((i < nbOfAlarms/2) ? "A_" : "B_") + i;
    if(!dpExists(dpName)){
      dpCreate(dpName, TEST_ALARM_DPE_TYPE);
      dyn_errClass deErrors = getLastError();
      if(dynlen(deErrors) > 0){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ + "(): failed to create DP '" +
                        dpName + "': " + getErrorText(deErrors), "");
        throwError(deErrors);
        return;
      }
      while(!dpExists(dpName)){
        delay(0, 100);
      }
    }

    dyn_mixed alarmObject;

    fwAlertConfig_objectCreateAnalog(
        alarmObject,
        makeDynString("COLD", "NORMAL", "HOT"),
        makeDynFloat(0, 22, 33),
        makeDynString("_fwWarningAck.", "", "_fwErrorAck."),
        "",
        makeDynString(""),
        "",
        makeDynBool(0, 1, 0),
        dsExceptions);
//"dist_1:TestAlarm_0.ProcessInput.dpeInt""dist_1:TestAlarm_0.ProcessInput"
    fwAlertConfig_objectSet(dpName + ".IntVal", alarmObject, dsExceptions);
    if(dynlen(dsExceptions) > 0){
      DebugN(dsExceptions);
    }
    fwAlertConfig_activate(dpName + ".IntVal", dsExceptions);
  }
}


void AsNgRegexParser_test_removeAlarms(int nbOfAlarms, dyn_string &dsExceptions){
  for(int i =0; i<nbOfAlarms; i++){
    string dpName = TEST_DPE_BASENAME + ((i < nbOfAlarms/2) ? "A_" : "B_") + i;
    if(dpExists(dpName)){
      dpDelete(dpName, TEST_ALARM_DPE_TYPE);
      dyn_errClass deErrors = getLastError();
      if(dynlen(deErrors) > 0){
        fwException_raise(dsExceptions, "ERROR", __FUNCTION__ + "(): failed to create DP '" +
                        dpName + "': " + getErrorText(deErrors), "");
        throwError(deErrors);
        return;
      }
    }
  }
}

void AsNgRegexParser_test_assertCount(int expectedCount, int actualCount){
  assert(actualCount == expectedCount, "Expected Count: " + expectedCount + "\nActual Count: " + actualCount);
}

void AsNgRegexParser_test_assertString(string expectedString, string actualString){
  assert(actualString == expectedString, "Expected String: " + expectedString + "\nActual String: " + actualString);

}

// test successful ewo creation
void test_AsNgRegexParser_EwoCreation() {
  shape ewo = AsNgRegexParser_test_getEwoShape();
  delay(10);
  AsNgRegexParser_test_assertCount(NO_OF_ALARMS, dynlen(ewo.getAllRowsData()));
}

void test_AsNgRegexParser_applyFilter(){
  shape ewo = AsNgRegexParser_test_getEwoShape();

  string sJson;
  dpGet(TEST_FILTER_DP + ".FilterJSON", sJson);

  ewo.applyFilter(sJson, "");
  delay(10);
  AsNgRegexParser_test_assertCount(NO_OF_ALARMS/2, dynlen(ewo.getAllRowsData()));
}
