/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"
#uses "classes/unCore/unBeep/UnBeepCPC6.ctl"
#uses "unDistributedControl/unDistributedControl.ctl"

const string UNBEEPCPC6_DP_ALIAS = "TEST01";
const string UNBEEPCPC6_DP = "TEST01-DP";
const string UNBEEPCPC6_DP_NAME = "TEST01-DP.";

const string UNBEEP_DP_ALIAS = "TestDistBeepState";
const string UNBEEP_DP = "unBeepDistDP";
const string UNBEEP_DP_NAME = "unBeepDistDP.";

dyn_string g_dsSystemName;
string g_bpType, g_bpDevice, g_bpPanel, g_bpTxt1, g_bpTxt2;

shared_ptr <UnBeep> beepCPC6ObjTest = nullptr;

//------------------------------------
// This routine is optional. It is never called, presence of this routine just tells to
// test runner that tests of this library shall run in UI manager
void test_unBeepCPC6_alwaysRunInUI()
{
}

void test_unBeepCPC6_setupSuite()
{
    string sHost;
    int iPort;

    // Set the Distributed Manager to always and start the manager
    unSystemIntegrity_getPmonHostAndPort(sHost, iPort);
    fwInstallationManager_setMode("WCCILdist", "", "always", sHost, iPort);
    fwInstallationManager_command("START", "WCCILdist", "", sHost, iPort);
    delay(60);

    // Store initial Beep configuration
    dpGet("_unApplication.beep.type", g_bpType,
          "_unApplication.beep.AlarmDevice", g_bpDevice,
          "_unApplication.beep.beepPanel", g_bpPanel,
          "_unApplication.beep.beepText1", g_bpTxt1,
          "_unApplication.beep.beepText2", g_bpTxt2);

    // Get distributed systems from config file
    dyn_string dsFileHostName, dsFileReduHost, exceptionInfo;
	dyn_int diFileSystemId, diFilePortNumber;

    unDistributedControl_getAllDeviceConfigFromFile(diFileSystemId, dsFileHostName, diFilePortNumber, dsFileReduHost, exceptionInfo);
    assertEqual(0, dynlen(exceptionInfo), "Could not extract system info from config file. Exception info: " + exceptionInfo);

    // Get list of systems that are connected to the distributed configuration
    dyn_uint dsSystemIds;
    getSystemNames(g_dsSystemName, dsSystemIds);

    dyn_string dsBpDevice;

    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      // Obtain sysId position of distributed system from config file
      int pos = dynContains(diFileSystemId, dsSystemIds[i]);
      if (pos > 0) {
        // Set the distributed system names and create the _unDistributedControl_systemName datapoint for distributed systems
        dyn_mixed systemInfo = makeDynMixed(g_dsSystemName[i], diFileSystemId[pos], dsFileHostName[pos], diFilePortNumber[pos], dsFileReduHost[pos]);
        unDistributedControl_addSystem(systemInfo, exceptionInfo);
        assertEqual(0, dynlen(exceptionInfo), "Could not create distributed system datapoint " + c_unDistributedControl_dpName + g_dsSystemName[i] + " . Exception info: " + exceptionInfo);
        delay(10);
        assertTrue(dpExists(c_unDistributedControl_dpName+g_dsSystemName[i]), "Distributed control datapoint for remote system " + g_dsSystemName[i] + " does not exists.");
      }

      // Add colon at the end of system names
      g_dsSystemName[i] += ":";

      // Check if the distributed system is connected
      bool isConnected;
      unDistributedControl_isConnected(isConnected, g_dsSystemName[i]);
      assertTrue(isConnected, "Distributed System " + g_dsSystemName[i] + " is not connected.");
      if(isConnected) {
          // Create Test BeepCPC6 datapoint with CPC_DigitalAlarm type for all the systems of the distributed configuration
          if(!dpExists(g_dsSystemName[i] + UNBEEPCPC6_DP)) {
            dpCreate(UNBEEPCPC6_DP, "CPC_DigitalAlarm", dsSystemIds[i]);
          }
          // Set the device alias if it is empty
          string alias = dpGetAlias(g_dsSystemName[i] + UNBEEPCPC6_DP_NAME);
          if(alias == "") {
            dpSetAlias(g_dsSystemName[i] + UNBEEPCPC6_DP_NAME, UNBEEPCPC6_DP_ALIAS);
          }
          // Append the device to the list of devices for the Application Configuration
          dynAppend(dsBpDevice, g_dsSystemName[i] + UNBEEPCPC6_DP_ALIAS);
      }
    }

    // Add unBeepCPC6 configuration to datapoint
    string bpPanel;
    string bpType = "UnBeepCPC6";
    string bpTxt1 = "Test";
    string bpTxt2 = "BeepCPC6";
    string bpDevice = strjoin(dsBpDevice, ",");

    dpSetWait("_unApplication.beep.type", bpType,
              "_unApplication.beep.AlarmDevice", bpDevice,
              "_unApplication.beep.beepPanel", bpPanel,
              "_unApplication.beep.beepText1", bpTxt1,
              "_unApplication.beep.beepText2", bpTxt2);

    // Deallocate BeepCPC6 Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }

    addSymbol(myModuleName(), myPanelName(), "objects/UN_GRAPHICALFRAME/unBeep.pnl", "BeepCPC6Obj", makeDynString(), -1, -1, 0, 0, 0);
    delay(2);
    shape beepShape = getShape("BeepCPC6Obj");

    // Initialize BeepCPC6 Test object
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);
}

void test_unBeepCPC6_teardownSuite()
{
    // Deallocate BeepCPC6 Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }

    // Load stored unBeep configuration
    dpSetWait("_unApplication.beep.type", g_bpType,
              "_unApplication.beep.AlarmDevice", g_bpDevice,
              "_unApplication.beep.beepPanel", g_bpPanel,
              "_unApplication.beep.beepText1", g_bpTxt1,
              "_unApplication.beep.beepText2", g_bpTxt2);

    // Delete Test BeepCPC6 Digital Alarm datapoints from current and distributed systems
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      if(dpExists(g_dsSystemName[i] + UNBEEPCPC6_DP)) {
        dpDelete(g_dsSystemName[i] + UNBEEPCPC6_DP);
      }
    }
}

void test_unBeepCPC6_AliasRenamed()
{
    // Rename alias to check automatic change in unBeep status
    dpSetAlias(UNBEEPCPC6_DP_NAME, "aliasRenamed");
    delay(5);
    assertEqual(UnBeepAlarmState::ConfigurationError, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Configuration Error caused by an alias renamed.");

    // Rename alias to original name and recheck automatic change in unBeep status
    dpSetAlias(UNBEEPCPC6_DP_NAME, UNBEEPCPC6_DP_ALIAS);
    delay(5);

    // Set values to OK
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
    }

    delay(5);
    assertEqual(UnBeepAlarmState::OK, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State should have returned to OK after renaming the alias back.");
}

void test_unBeepCPC6_DpCreationDeletion()
{
    // Delete unBeep datapoint to check automatic change in unBeep status
    if(dpExists(UNBEEPCPC6_DP)) {
      dpDelete(UNBEEPCPC6_DP);
    }
    delay(5);
    assertEqual(UnBeepAlarmState::ConfigurationError, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Configuration Error caused by a DP deletion.");

    // Create previous deleted Beep datapoint
    if(!dpExists(UNBEEPCPC6_DP)) {
      dpCreate(UNBEEPCPC6_DP, "CPC_DigitalAlarm");
    }

    dpSetAlias(UNBEEPCPC6_DP_NAME, UNBEEPCPC6_DP_ALIAS);
    delay(5);

    // Set values to OK
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
    }

    delay(5);
    assertEqual(UnBeepAlarmState::OK, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State should have returned to OK after creating the DP and renaming the alias back.");
}

void test_unBeepCPC6_OKState()
{
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
    }

    delay(2);
    assertEqual(UnBeepAlarmState::OK, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than OK");
}

void test_unBeepCPC6_AlarmMaskedState()
{
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
      if(i == dynlen(g_dsSystemName)) {
          bit32 b;
          setBit(b, UN_STSREG01_ALMSKST, TRUE);
          dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", b);
      } else {
          dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      }
    }

    delay(2);
    assertEqual(UnBeepAlarmState::AlarmMasked, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Alarm Masked");
}

void test_unBeepCPC6_AlarmNotAckState()
{
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
      if(i == dynlen(g_dsSystemName)) {
          bit32 b;
          setBit(b, UN_STSREG01_ALUNACK, TRUE);
          dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", b);
      } else {
          dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      }
    }

    delay(2);
    assertEqual(UnBeepAlarmState::AlarmNotAck, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Alarm Not Acknowledge");
}

void test_unBeepCPC6_AlarmActiveState()
{
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
      if(i == dynlen(g_dsSystemName)) {
          bit32 b;
          setBit(b, UN_STSREG01_POSST, TRUE);
          dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", b);
      } else {
          dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      }
    }

    delay(2);
    assertEqual(UnBeepAlarmState::AlarmActive, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Alarm Active");
}

void test_unBeepCPC6_InvalidState()
{
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      if(i == dynlen(g_dsSystemName)) {
          dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", TRUE);
      }
    }

    delay(2);
    assertEqual(UnBeepAlarmState::InvalidData, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Invalid Data");
}

void test_unBeepCPC6_DistSystemsNotConnected()
{
    assertFalse(dynlen(g_dsSystemName) == 1 && dynContains(g_dsSystemName, getSystemName()), "test_unBeepCPC6_DistSystemsNotConnected() failed because only the current " +
                "system is connected. Systems connected: " + g_dsSystemName);

    string sHost;
    int iPort;
    unSystemIntegrity_getPmonHostAndPort(sHost, iPort);
    fwInstallationManager_setMode("WCCILdist", "", "manual", sHost, iPort);
    fwInstallationManager_command("KILL", "WCCILdist", "",  sHost, iPort);
    delay(5);

    assertEqual(UnBeepAlarmState::ConfigurationError, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Configuration Error caused by dist manager disconnection.");

    fwInstallationManager_setMode("WCCILdist", "", "always", sHost, iPort);
    fwInstallationManager_command("START", "WCCILdist", "", sHost, iPort);
    delay(60);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    string bpType = "UnBeepCPC6";
    shape beepShape = getShape("BeepCPC6Obj");
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);

    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
    }

    delay(5);
    assertEqual(UnBeepAlarmState::OK, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State should have returned to OK after dist manager connection.");
}

void test_unBeepCPC6_ConfigurationError()
{
    dyn_string dsBpDevice;

    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      if (i == dynlen(g_dsSystemName)) {
        dynAppend(dsBpDevice, g_dsSystemName[i] + UNBEEPCPC6_DP_ALIAS + "_NotExisting");
      } else {
        dynAppend(dsBpDevice, g_dsSystemName[i] + UNBEEPCPC6_DP_ALIAS);
      }
    }

    // Add Configuration Error unBeep to datapoint
    string bpDevice = strjoin(dsBpDevice, ",");
    dpSetWait("_unApplication.beep.AlarmDevice", bpDevice);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    string bpType = "UnBeepCPC6";
    shape beepShape = getShape("BeepCPC6Obj");
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepAlarmState::ConfigurationError, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Configuration Error caused by incorrect device string.");

    // Fix configuration error and set status to OK
    dynClear(dsBpDevice);

    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dynAppend(dsBpDevice, g_dsSystemName[i] + UNBEEPCPC6_DP_ALIAS);
    }

    string bpDevice = strjoin(dsBpDevice, ",");
    dpSetWait("_unApplication.beep.AlarmDevice", bpDevice);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);

    // Set values to OK
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
    }

    delay(5);
    assertEqual(UnBeepAlarmState::OK, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State should have returned to OK after device string configuration error.");
}

void test_unBeepCPC6_IncorrectBeepType()
{
    string bpType = "UnBeep";
    dpSetWait("_unApplication.beep.type", bpType);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    shape beepShape = getShape("BeepCPC6Obj");
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepAlarmState::ConfigurationError, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Configuration Error caused by incorrect Beep type.");

    // Set Beep type back to UnBeep again
    bpType = "UnBeepCPC6";
    dpSetWait("_unApplication.beep.type", bpType);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);

    // Set values to OK
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
    }

    delay(5);
    assertEqual(UnBeepAlarmState::OK, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State should have returned to OK after Beep type configuration error.");
}

void test_unBeepCPC6_NotConfigured()
{
    // Add Not Cofigured unBeep to datapoint
    dpSetWait("_unApplication.beep.AlarmDevice", "");

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    string bpType = "UnBeepCPC6";
    shape beepShape = getShape("BeepCPC6Obj");
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepAlarmState::NotConfigured, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Not Configured.");

    // Fix not configured issue and set status to OK
    dyn_string dsBpDevice;
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dynAppend(dsBpDevice, g_dsSystemName[i] + UNBEEPCPC6_DP_ALIAS);
    }

    string bpDevice = strjoin(dsBpDevice, ",");
    dpSetWait("_unApplication.beep.AlarmDevice", bpDevice);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);

    // Set values to OK
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
    }

    delay(5);
    assertEqual(UnBeepAlarmState::OK, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State should have returned to OK after not configuration.");
}

void test_unBeepCPC6_MixedAliasesTypes()
{
    // Create Example Beep datapoint for current system
    if(!dpExists(UNBEEP_DP)) {
      dpCreate(UNBEEP_DP, "ExampleDP_Bit");
    }

    string alias = dpGetAlias(UNBEEP_DP_NAME);
    if(alias == "") {
      dpSetAlias(UNBEEP_DP_NAME, UNBEEP_DP_ALIAS);
    }

    dyn_string dsBpDevice = makeDynString(getSystemName()+UNBEEP_DP_ALIAS);

    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dynAppend(dsBpDevice, g_dsSystemName[i] + UNBEEPCPC6_DP_ALIAS);
    }

    // Add mixed aliases types Configuration Error unBeep to datapoint
    string bpDevice = strjoin(dsBpDevice, ",");
    dpSetWait("_unApplication.beep.AlarmDevice", bpDevice);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    string bpType = "UnBeepCPC6";
    shape beepShape = getShape("BeepCPC6Obj");
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepAlarmState::ConfigurationError, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State is different than Configuration Error caused by mixed aliases types (default and CPC6).");

    if(dpExists(getSystemName() + UNBEEP_DP)) {
      dpDelete(getSystemName() + UNBEEP_DP);
    }

    // Remove Default alias from alarm device dpe
    dynRemove(dsBpDevice, 1);
    bpDevice = strjoin(dsBpDevice, ",");
    dpSetWait("_unApplication.beep.AlarmDevice", bpDevice);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCPC6ObjTest != nullptr) {
      beepCPC6ObjTest.finalize();
      beepCPC6ObjTest = nullptr;
    }
    // Initialize Beep Test object
    beepCPC6ObjTest = UnBeep::getInstance(bpType);
    beepCPC6ObjTest.setIsUnitTests(true);
    beepCPC6ObjTest.initialize(beepShape);
    delay(5);

    // Set values to OK
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01", 0);
      dpSetWait(g_dsSystemName[i] + UNBEEPCPC6_DP + ".ProcessInput.StsReg01:_original.._exp_inv", FALSE);
    }

    delay(5);
    assertEqual(UnBeepAlarmState::OK, beepCPC6ObjTest.getAlarmState(), "Beep Alarm State should have returned to OK after Beep type configuration error caused by mixed aliases types (default and CPC6).");
}
