/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"
#uses "classes/unCore/unBeep/UnBeepCryo.ctl"
#uses "unDistributedControl/unDistributedControl.ctl"

const dyn_string UNBEEPCRYO_DP_ALIAS_LIST = makeDynString("DevicePos1_Obsolete", "TEST01_Left", "TEST02_Left", "TEST03_Left", "TEST0nOff_Left",
                                                          "TEST01_Right", "TEST02_Right", "TEST03_Right", "TESTOnOff_Right");
const dyn_string UNBEEPCRYO_DP_LIST = makeDynString("DevicePos1-DP_Obsolete", "TEST01-DP_Left", "TEST02-DP_Left", "TEST03-DP_Left", "TESTOnOff-DP_Left",
                                                    "TEST01-DP_Right", "TEST02-DP_Right", "TEST03-DP_Right", "TESTOnOff-DP_Right");
const dyn_string UNBEEPCRYO_DP_NAME_LIST = makeDynString("DevicePos1-DP_Obsolete.", "TEST01-DP_Left.", "TEST02-DP_Left.", "TEST03-DP_Left.", "TESTOnOff-DP_Left.",
                                                         "TEST01-DP_Right.", "TEST02-DP_Right.", "TEST03-DP_Right.", "TESTOnOff-DP_Right.");

dyn_string g_dsSystemName;
string g_testSystemNameLeft_1, g_testSystemNameLeft_2, g_testSystemNameRight_1, g_testSystemNameRight_2;
string g_bpType, g_bpDevice, g_bpPanel, g_bpTxt1, g_bpTxt2;

shared_ptr <UnBeep> beepCryoObjTest = nullptr;

//------------------------------------
// This routine is optional. It is never called, presence of this routine just tells to
// test runner that tests of this library shall run in UI manager
void test_unBeepCryo_alwaysRunInUI()
{
}

void test_unBeepCryo_setupSuite()
{
    string sHost;
    int iPort;

    // Set the Distributed Manager to always and start the manager
    unSystemIntegrity_getPmonHostAndPort(sHost, iPort);
    fwInstallationManager_setMode("WCCILdist", "", "always", sHost, iPort);
    fwInstallationManager_command("START", "WCCILdist", "", sHost, iPort);
    delay(60);

    // Store initial Beep configuration
    dpGet("_unApplication.beep.type", g_bpType,
          "_unApplication.beep.AlarmDevice", g_bpDevice,
          "_unApplication.beep.beepPanel", g_bpPanel,
          "_unApplication.beep.beepText1", g_bpTxt1,
          "_unApplication.beep.beepText2", g_bpTxt2);

    // Get distributed systems from config file
    dyn_string dsFileHostName, dsFileReduHost, exceptionInfo;
	dyn_int diFileSystemId, diFilePortNumber;

    unDistributedControl_getAllDeviceConfigFromFile(diFileSystemId, dsFileHostName, diFilePortNumber, dsFileReduHost, exceptionInfo);
    assertEqual(0, dynlen(exceptionInfo), "Could not extract system info from config file. Exception info: " + exceptionInfo);

    // Get list of systems that are connected to the distributed configuration
    dyn_uint dsSystemIds;
    getSystemNames(g_dsSystemName, dsSystemIds);

    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      // Obtain sysId position of distributed system from config file
      int pos = dynContains(diFileSystemId, dsSystemIds[i]);
      if (pos > 0) {
        // Set the distributed system names and create the _unDistributedControl_systemName datapoint for distributed systems
        dyn_mixed systemInfo = makeDynMixed(g_dsSystemName[i], diFileSystemId[pos], dsFileHostName[pos], diFilePortNumber[pos], dsFileReduHost[pos]);
        unDistributedControl_addSystem(systemInfo, exceptionInfo);
        assertEqual(0, dynlen(exceptionInfo), "Could not create distributed system datapoint " + c_unDistributedControl_dpName + g_dsSystemName[i] + " . Exception info: " + exceptionInfo);
        delay(10);
        assertTrue(dpExists(c_unDistributedControl_dpName+g_dsSystemName[i]), "Distributed control datapoint for remote system " + g_dsSystemName[i] + " does not exists.");
      }

      // Add colon at the end of system names
      g_dsSystemName[i] += ":";

      // Check if the distributed system is connected
      bool isConnected;
      unDistributedControl_isConnected(isConnected, g_dsSystemName[i]);
      assertTrue(isConnected, "Distributed System " + g_dsSystemName[i] + " is not connected.");
      if(isConnected) {
        // Create Test unBeepCryo datapoints with CPC_DigitalAlarm or CPC_OnOff type for all the systems of the distributed configuration
        for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
          if(!dpExists(g_dsSystemName[i] + UNBEEPCRYO_DP_LIST[j])) {
            if (j != 5 && j != 9) {
              dpCreate(UNBEEPCRYO_DP_LIST[j], "CPC_DigitalAlarm", dsSystemIds[i]);
            }
            else {
              dpCreate(UNBEEPCRYO_DP_LIST[j], "CPC_OnOff", dsSystemIds[i]);
            }
          }
          // Set the device alias if it is empty
          string alias = dpGetAlias(g_dsSystemName[i] + UNBEEPCRYO_DP_NAME_LIST[j]);
          if(alias == "") {
            dpSetAlias(g_dsSystemName[i] + UNBEEPCRYO_DP_NAME_LIST[j], UNBEEPCRYO_DP_ALIAS_LIST[j]);
          }
        }
      }
    }

    // Append the device to the list of devices for the Application Configuration datapoint
    dyn_string dsBpDevice;
    if (dynlen(g_dsSystemName) == 1) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        g_testSystemNameLeft_1 = g_dsSystemName[1];
        g_testSystemNameLeft_2 = g_dsSystemName[1];
        g_testSystemNameRight_1 = g_dsSystemName[1];
        g_testSystemNameRight_2 = g_dsSystemName[1];
      }
    }
    else if (dynlen(g_dsSystemName) > 1 && dynlen(g_dsSystemName) < 4) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        if (j < 6) {
          dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else {
          dynAppend(dsBpDevice, g_dsSystemName[2] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
      }
      g_testSystemNameLeft_1 = g_dsSystemName[1];
      g_testSystemNameLeft_2 = g_dsSystemName[1];
      g_testSystemNameRight_1 = g_dsSystemName[2];
      g_testSystemNameRight_2 = g_dsSystemName[2];
    }
    else if (dynlen(g_dsSystemName) >= 4) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        if (j < 4) {
          dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 4 && j < 6) {
          dynAppend(dsBpDevice, g_dsSystemName[2] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 6 && j < 8) {
          dynAppend(dsBpDevice, g_dsSystemName[3] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 8) {
          dynAppend(dsBpDevice, g_dsSystemName[4] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
      }
      g_testSystemNameLeft_1 = g_dsSystemName[1];
      g_testSystemNameLeft_2 = g_dsSystemName[2];
      g_testSystemNameRight_1 = g_dsSystemName[3];
      g_testSystemNameRight_2 = g_dsSystemName[4];
    }

    // Add unBeepCryo configuration to datapoint
    string bpPanel;
    string bpType = "UnBeepCryo";
    string bpTxt1 = "Test";
    string bpTxt2 = "BeepCryo";
    string bpDevice = strjoin(dsBpDevice, ",");

    dpSetWait("_unApplication.beep.type", bpType,
              "_unApplication.beep.AlarmDevice", bpDevice,
              "_unApplication.beep.beepPanel", bpPanel,
              "_unApplication.beep.beepText1", bpTxt1,
              "_unApplication.beep.beepText2", bpTxt2);

    // Deallocate BeepCryo Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }

    addSymbol(myModuleName(), myPanelName(), "objects/UN_GRAPHICALFRAME/unBeep.pnl", "BeepCryoObj", makeDynString(), -1, -1, 0, 0, 0);
    delay(2);
    shape beepShape = getShape("BeepCryoObj");

    // Initialize BeepCryo Test object
    beepCryoObjTest = UnBeep::getInstance(bpType);
    beepCryoObjTest.setIsUnitTests(true);
    beepCryoObjTest.initialize(beepShape);
    delay(5);
}

void test_unBeepCryo_teardownSuite()
{
    // Deallocate BeepCryo Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }

    // Load stored unBeep configuration
    dpSetWait("_unApplication.beep.type", g_bpType,
              "_unApplication.beep.AlarmDevice", g_bpDevice,
              "_unApplication.beep.beepPanel", g_bpPanel,
              "_unApplication.beep.beepText1", g_bpTxt1,
              "_unApplication.beep.beepText2", g_bpTxt2);

    // Delete Test BeepCryo Digital Alarm datapoints from current and distributed systems
    for(int i = 1; i <= dynlen(g_dsSystemName); i++) {
      // Create Test unBeepCryo datapoints with CPC_DigitalAlarm or CPC_OnOff type for all the systems of the distributed configuration
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        if(dpExists(g_dsSystemName[i] + UNBEEPCRYO_DP_LIST[j])) {
          dpDelete(g_dsSystemName[i] + UNBEEPCRYO_DP_LIST[j]);
        }
      }
    }
}

void test_unBeepCryo_AliasRenamed()
{
    // Initialize the Beep boxes to Init State
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 0);

    // Rename alias to check automatic change in Beep Box Left status
    dpSetAlias(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_NAME_LIST[2], "aliasRenamed");
    delay(5);
    assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Configuration Error caused by an alias renamed.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init.");

    // Rename alias to original name and recheck automatic change in unBeep status
    dpSetAlias(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_NAME_LIST[2], UNBEEPCRYO_DP_ALIAS_LIST[2]);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init caused by an alias renamed.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init.");

    // Rename alias to check automatic change in Beep Box Right status
    dpSetAlias(g_testSystemNameRight_1 + UNBEEPCRYO_DP_NAME_LIST[6], "aliasRenamed");
    delay(5);
    assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Configuration Error caused by an alias renamed.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");

    // Rename alias to original name and recheck automatic change in unBeep status
    dpSetAlias(g_testSystemNameRight_1 + UNBEEPCRYO_DP_NAME_LIST[6], UNBEEPCRYO_DP_ALIAS_LIST[6]);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init caused by an alias renamed.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");
}

void test_unBeepCryo_DpCreationDeletion()
{
    // Initialize the Beep boxes to Init State
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 0);

    // Delete unBeep datapoint to check automatic change in Beep Box Left status
    if(dpExists(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2])) {
      dpDelete(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2]);
    }
    delay(5);

    assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Configuration Error caused by a DP deletion.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init.");

    int sysId = getSystemId(g_testSystemNameLeft_1);
    // Create previous deleted Beep datapoint
    if(!dpExists(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2])) {
      dpCreate(UNBEEPCRYO_DP_LIST[2], "CPC_DigitalAlarm", sysId);
    }

    dpSetAlias(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_NAME_LIST[2], UNBEEPCRYO_DP_ALIAS_LIST[2]);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init caused by a DP creation.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init.");


    // Delete unBeep datapoint to check automatic change in Beep Box Right status
    if(dpExists(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6])) {
      dpDelete(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6]);
    }
    delay(5);

    assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Configuration Error caused by a DP deletion.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");

    sysId = getSystemId(g_testSystemNameRight_1);
    // Create previous deleted Beep datapoint
    if(!dpExists(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6])) {
      dpCreate(UNBEEPCRYO_DP_LIST[6], "CPC_DigitalAlarm", sysId);
    }

    dpSetAlias(g_testSystemNameRight_1 + UNBEEPCRYO_DP_NAME_LIST[6], UNBEEPCRYO_DP_ALIAS_LIST[6]);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init caused by a DP creation.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");
}

void test_unBeepCryo_InitState()
{
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after setting StsReg01.");
}

void test_unBeepCryo_OnState()
{
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 1);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 1);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::OnState, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than OnState after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Left Box is different than Init.");

    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 1);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 1);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::OnState, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than OnState after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after setting StsReg01.");

    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 0);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");
}

void test_unBeepCryo_OffState()
{
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 1);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 2);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::OffState, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than OffState after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Left Box is different than Init.");

    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 1);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 2);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::OffState, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than OffState after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after setting StsReg01.");

    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 0);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");
}

void test_unBeepCryo_AlarmBlocked_OnState()
{
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 1000000);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 1);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::AlarmBlocked_OnState, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than AlarmBlocked_OnState after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Left Box is different than Init.");

    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 1000000);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 1);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::AlarmBlocked_OnState, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than AlarmBlocked_OnState after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after setting StsReg01.");

    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 0);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");
}

void test_unBeepCryo_AlarmBlocked_OffState()
{
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 1000000);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 2);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::AlarmBlocked_OffState, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than AlarmBlocked_OffState after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Left Box is different than Init.");

    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 1000000);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 2);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::AlarmBlocked_OffState, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than AlarmBlocked_OffState after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after setting StsReg01.");

    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 0);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");
}

void test_unBeepCryo_AlarmActiveState()
{
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", -1);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 1);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::AlarmActive, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than AlarmActive after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Left Box is different than Init.");

    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", -1);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 1);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::AlarmActive, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than AlarmActive after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after setting StsReg01.");

    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 0);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");
}

void test_unBeepCryo_StopInterlockState()
{
    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", -1);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::StopInterlock, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than StopInterlock after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Left Box is different than Init.");

    dpSetWait(g_testSystemNameLeft_1 + UNBEEPCRYO_DP_LIST[2] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameLeft_2 + UNBEEPCRYO_DP_LIST[5] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", -1);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::StopInterlock, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than StopInterlock after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after setting StsReg01.");

    dpSetWait(g_testSystemNameRight_1 + UNBEEPCRYO_DP_LIST[6] + ".ProcessInput.StsReg01", 0);
    dpSetWait(g_testSystemNameRight_2 + UNBEEPCRYO_DP_LIST[9] + ".ProcessInput.StsReg01", 0);
    delay(2);
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after setting StsReg01.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init.");
}

// Test is currently commented because it does not work. The issue is that when the Distributed Manager gets disconnected, the _UnDistributedControl
// datapoint element that returns the connection status (.connected) might be incorrect for the distributed systems
/*void test_unBeepCryo_DistSystemsNotConnected()
{
    assertFalse(dynlen(g_dsSystemName) == 1 && dynContains(g_dsSystemName, getSystemName()), "test_unBeepCryo_DistSystemsNotConnected() failed because only the current " +
                "system is connected. Systems connected: " + g_dsSystemName);

    string sHost;
    int iPort;

    unSystemIntegrity_getPmonHostAndPort(sHost, iPort);
    fwInstallationManager_setMode("WCCILdist", "", "manual", sHost, iPort);
    fwInstallationManager_command("KILL", "WCCILdist", "", sHost, iPort);
    delay(5);

    if (dynlen(g_dsSystemName) < 4) {
      if (g_testSystemNameLeft_1 != getSystemName()) {
        assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Configuration Error caused by dist manager disconnection.");
      }
      if (g_testSystemNameRight_1 != getSystemName()) {
        assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Configuration Error caused by dist manager disconnection.");
      }
    }
    else if (dynlen(g_dsSystemName) >= 4) {
        assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Configuration Error caused by dist manager disconnection.");
        assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Configuration Error caused by dist manager disconnection.");
    }

    fwInstallationManager_setMode("WCCILdist", "", "always", sHost, iPort);
    fwInstallationManager_command("START", "WCCILdist", "", sHost, iPort);
    delay(60);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }
    // Initialize Beep Test object
    string bpType = "UnBeepCryo";
    shape beepShape = getShape("BeepCryoObj");
    beepCryoObjTest = UnBeep::getInstance(bpType);
    beepCryoObjTest.setIsUnitTests(true);
    beepCryoObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after resetting AlarmDevice Beep datapoint element.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after resetting AlarmDevice Beep datapoint element.");
} */

void test_unBeepCryo_ConfigurationError()
{
    // Append the device to the list of devices for the Application Configuration datapoint
    dyn_string dsBpDevice;
    for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
      // First device alias does not matter because it is never checked and used anywhere for unBeep Cryo
      if (j <= 2) {
        dynAppend(dsBpDevice, g_testSystemNameLeft_1 + UNBEEPCRYO_DP_ALIAS_LIST[j] + "_NotExisting");
      }
      else if (j > 2 && j < 6) {
        dynAppend(dsBpDevice, g_testSystemNameLeft_1 + UNBEEPCRYO_DP_ALIAS_LIST[j]);
      }
      else if (j == 6) {
        dynAppend(dsBpDevice, g_testSystemNameRight_1 + UNBEEPCRYO_DP_ALIAS_LIST[j] + "_NotExisting");
      }
      else if (j > 6) {
        dynAppend(dsBpDevice, g_testSystemNameRight_1 + UNBEEPCRYO_DP_ALIAS_LIST[j]);
      }
    }

    // Add Configuration Error unBeep to datapoint
    string bpDevice = strjoin(dsBpDevice, ",");
    dpSetWait("_unApplication.beep.AlarmDevice", bpDevice);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }
    // Initialize Beep Test object
    string bpType = "UnBeepCryo";
    shape beepShape = getShape("BeepCryoObj");
    beepCryoObjTest = UnBeep::getInstance(bpType);
    beepCryoObjTest.setIsUnitTests(true);
    beepCryoObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than ConfigurationError after setting a device alias that does not exist.");
    assertEqual(UnBeepCryoAlarmState::ConfigurationError, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than ConfigurationError after setting a device alias that does not exist.");

    // Fix configuration error and set status to OK
    dynClear(dsBpDevice);

    if (dynlen(g_dsSystemName) == 1) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
      }
    }
    else if (dynlen(g_dsSystemName) > 1 && dynlen(g_dsSystemName) < 4) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        if (j < 6) {
          dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else {
          dynAppend(dsBpDevice, g_dsSystemName[2] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
      }
    }
    else if (dynlen(g_dsSystemName) >= 4) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        if (j < 4) {
          dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 4 && j < 6) {
          dynAppend(dsBpDevice, g_dsSystemName[2] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 6 && j < 8) {
          dynAppend(dsBpDevice, g_dsSystemName[3] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 8) {
          dynAppend(dsBpDevice, g_dsSystemName[4] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
      }
    }

    string bpDevice = strjoin(dsBpDevice, ",");
    dpSetWait("_unApplication.beep.AlarmDevice", bpDevice);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }
    // Initialize Beep Test object
    beepCryoObjTest = UnBeep::getInstance(bpType);
    beepCryoObjTest.setIsUnitTests(true);
    beepCryoObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after resetting AlarmDevice Beep datapoint element.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after resetting AlarmDevice Beep datapoint element.");
}

void test_unBeepCryo_IncorrectBeepType()
{
    string bpType = "UnBeep";
    dpSetWait("_unApplication.beep.type", bpType);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }
    // Initialize Beep Test object
    shape beepShape = getShape("BeepCryoObj");
    beepCryoObjTest = UnBeep::getInstance(bpType);
    beepCryoObjTest.setIsUnitTests(true);
    beepCryoObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepAlarmState::ConfigurationError, beepCryoObjTest.getAlarmState(), "Beep Alarm State is different than Configuration Error caused by incorrect Beep type.");

    // Set Beep type back to UnBeep again
    bpType = "UnBeepCryo";
    dpSetWait("_unApplication.beep.type", bpType);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }
    // Initialize Beep Test object
    beepCryoObjTest = UnBeep::getInstance(bpType);
    beepCryoObjTest.setIsUnitTests(true);
    beepCryoObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box is different than Init after resetting Beep type datapoint element.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box is different than Init after resetting Beep type datapoint element.");
}

void test_unBeepCryo_NotConfigured()
{
    // Add Not Cofigured unBeep to datapoint
    dpSetWait("_unApplication.beep.AlarmDevice", "");

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }
    // Initialize Beep Test object
    string bpType = "UnBeepCryo";
    shape beepShape = getShape("BeepCryoObj");
    beepCryoObjTest = UnBeep::getInstance(bpType);
    beepCryoObjTest.setIsUnitTests(true);
    beepCryoObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepAlarmState::NotConfigured, beepCryoObjTest.getAlarmState(), "Beep Cryo Alarm State is different than Not Configured.");

    // Fix not configured issue and set status to OK
    // Append the device to the list of devices for the Application Configuration datapoint
    dyn_string dsBpDevice;
    if (dynlen(g_dsSystemName) == 1) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
      }
    }
    else if (dynlen(g_dsSystemName) > 1 && dynlen(g_dsSystemName) < 4) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        if (j < 6) {
          dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else {
          dynAppend(dsBpDevice, g_dsSystemName[2] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
      }
    }
    else if (dynlen(g_dsSystemName) >= 4) {
      for (int j = 1; j <= dynlen(UNBEEPCRYO_DP_LIST); j++) {
        if (j < 4) {
          dynAppend(dsBpDevice, g_dsSystemName[1] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 4 && j < 6) {
          dynAppend(dsBpDevice, g_dsSystemName[2] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 6 && j < 8) {
          dynAppend(dsBpDevice, g_dsSystemName[3] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
        else if (j >= 8) {
          dynAppend(dsBpDevice, g_dsSystemName[4] + UNBEEPCRYO_DP_ALIAS_LIST[j]);
        }
      }
    }

    string bpDevice = strjoin(dsBpDevice, ",");
    dpSetWait("_unApplication.beep.AlarmDevice", bpDevice);

    // Change in configuration - reset Beep
    // Deallocate Beep Test object if it exists
    if (beepCryoObjTest != nullptr) {
      beepCryoObjTest.finalize();
      beepCryoObjTest = nullptr;
    }
    // Initialize Beep Test object
    beepCryoObjTest = UnBeep::getInstance(bpType);
    beepCryoObjTest.setIsUnitTests(true);
    beepCryoObjTest.initialize(beepShape);
    delay(5);

    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepR"), "Beep Cryo Alarm State for Right Box should have returned to Init after not configuration.");
    assertEqual(UnBeepCryoAlarmState::Init, beepCryoObjTest.getCryoAlarmState("BoxBeepL"), "Beep Cryo Alarm State for Left Box should have returned to Init after not configuration.");
}
