/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"

#uses "unExportDevice.ctl"


const string TEST_S7_CONNECTION_NAME = "S7_Conn_TEST";
const string TEST_S7_CONNECTION_DP = "_"+TEST_S7_CONNECTION_NAME;

//-------------------------------------------------------
void test_unExportDevice_FrontEnd_S7_setupSuite()
{
    dpCreate(TEST_S7_CONNECTION_DP, "_S7_Conn");
    assert(dpExists(TEST_S7_CONNECTION_DP)==true, "Internal S7 connection DP could not be created.");
}

void _setup_S7_connection_TSPP_only(){
    dyn_string dpes;
    dyn_mixed values;

    dpes[1] = TEST_S7_CONNECTION_DP + ".SetInvalidBit";
    values[1] = FALSE;

    dpes[2] = TEST_S7_CONNECTION_DP + ".UseTSPP";
    values[2] = TRUE;

    dpes[3] = TEST_S7_CONNECTION_DP + ".AlarmActive";
    values[3] = FALSE;

    dpes[4] = TEST_S7_CONNECTION_DP + ".EnableDiagnostics";
    values[4] = FALSE;

    dpes[5] = TEST_S7_CONNECTION_DP + ".DrvNumber";
    values[5] = 2;

    dpes[6] = TEST_S7_CONNECTION_DP + ".Active";
    values[6] = FALSE;

    dpes[7] = TEST_S7_CONNECTION_DP + ".Address";
    values[7] = "{\"TSPPExtras\":\"1.16.0.0.16.0\",\"ConnectionType\":3,\"TimeOut\":5000,\"Slot\":2,\"ProtocolExtras\":\"\",\"Rack\":0,\"IPAddress\":\"137.138.25.115\"}";

    int res = dpSetWait(dpes, values);
    dyn_errClass err = getLastError();
    assert(res==0 && dynlen(err)==0, "Internal S7 connection DP not initialized properly: "+(string)err);
}

//-------------------------------------------------------
void test_unExportDevice_FrontEnd_S7_connectionDefinition()
{
    _setup_S7_connection_TSPP_only();

    dyn_string dsConfig;
    S7_PLC_getConfigToExport(TEST_S7_CONNECTION_NAME, dsConfig);

    assertEqual(dsConfig[1], 1, "TSPP connection ID is wrong.");
    assertEqual(dsConfig[2], 0, "Rack number is wrong.");
    assertEqual(dsConfig[3], 2, "Connection slot is wrong.");
    assertEqual(dsConfig[4], 10, "TSPP Cp Id is wrong.");
    assertEqual(dsConfig[5], 0, "TSPP Pc Rack is wrong.");
    assertEqual(dsConfig[6], 0, "TSPP Pc Slot is wrong.");
    assertEqual(dsConfig[7], 10, "TSPP Pc Id is wrong.");
    assertEqual(dsConfig[8], 5000, "Timeout is wrong.");
    assertEqual(dsConfig[9], "137.138.25.115", "IP is wrong.");
}

//------------------------------------------------------
void test_unExportDevice_FrontEnd_S7_teardownSuite()
{
    dpDelete(TEST_S7_CONNECTION_DP);
}
