/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"

//------------------------------------
// This routine is optional. It is never called, presence of this routine just tells to
// test runner that tests of this library shall run in UI manager
void test_unFileAccessControlConfiguration_alwaysRunInUI()
{
}

//------------------------------------------------------
void test_unFileAccessControlConfiguration_setupSuite() {
}


//------------------------------------------------------
void test_unFileAccessControlConfiguration_addConfig() {
	
	dyn_string dsRes;
	dyn_float dfRes;

	string sFileName = "vision/test/unCore/unFileAccessControlConfiguration/test_unFileAccessControlConfiguration.pnl";
	string sDomain = "TestDomain";
	string sAccess = "Expert";
	string sConfigLine = sFileName + "|" + sDomain + "||Access" + sAccess + "||";

	dyn_string dsMenuAccess;
	dpGet("_unApplication.menu.menuFileAccessControl", dsMenuAccess);
	int iOldPos = dynlen(dsMenuAccess);
	
	// Remove test File Access Control Configuration entry if it already exists
	if(dsMenuAccess[iOldPos].contains(sFileName)) {
	  test_unFileAccessControlConfiguration_removeConfig();
	  dpGet("_unApplication.menu.menuFileAccessControl", dsMenuAccess);
	  iOldPos = dynlen(dsMenuAccess);
	}

	ChildPanelOnReturn("vision/test/unCore/unFileAccessControlConfiguration/test_unFileAccessControlConfiguration_addConfig.xml",
								"testAddConfig",
								makeDynString("$sFileName:"+sFileName, "$sDomain:"+sDomain, "$sAccess:"+sAccess),
								-1, -1,
								dfRes,
								dsRes);

	// Check that a new line has been added to the list of file access control configurations
	dpGet("_unApplication.menu.menuFileAccessControl", dsMenuAccess);
	int iNewPos = dynlen(dsMenuAccess);
	assertEqual(iOldPos+1, iNewPos, "The length of the configuration after adding a new line is incorrect");
	assertEqual(dsMenuAccess[iNewPos], sConfigLine, "New added configuration line is incorrect");
}

//---------------------------------------------------------
void test_unFileAccessControlConfiguration_removeConfig() {
	
	dyn_string dsRes;
	dyn_float dfRes;
	
	string sFileName = "vision/test/unCore/unFileAccessControlConfiguration/test_unFileAccessControlConfiguration.pnl";

	dyn_string dsMenuAccess;
	dpGet("_unApplication.menu.menuFileAccessControl", dsMenuAccess);
	int iOldPos = dynlen(dsMenuAccess);

	// Add test File Access Control Configuration entry if it does not exist
	if(!dsMenuAccess[iOldPos].contains(sFileName)) {
	  test_unFileAccessControlConfiguration_addConfig();
	  dpGet("_unApplication.menu.menuFileAccessControl", dsMenuAccess);
	  iOldPos = dynlen(dsMenuAccess);
	}

	ChildPanelOnReturn("vision/test/unCore/unFileAccessControlConfiguration/test_unFileAccessControlConfiguration_removeConfig.xml",
								"testRemoveConfig",
								makeDynString(""),
								-1, -1,
								dfRes,
								dsRes);

	// Check that the last line of the list of file access control configurations has been removed
	dpGet("_unApplication.menu.menuFileAccessControl", dsMenuAccess);
	int iNewPos = dynlen(dsMenuAccess);
	assertEqual(iOldPos-1, iNewPos, "The length of the configuration after removing the last line is incorrect");
}

//------------------------------------------------------
void test_unFileAccessControlConfiguration_teardownSuite() {
	dyn_string dsMenuAccess;
	string sFileName = "vision/test/unCore/unFileAccessControlConfiguration/test_unFileAccessControlConfiguration.pnl";
	dpGet("_unApplication.menu.menuFileAccessControl", dsMenuAccess);
	int iOldPos = dynlen(dsMenuAccess);
	// Remove test File Access Control Configuration entry if it exists before closing
	// the unFileAccessControlConfiguration unit test suite
	if(dsMenuAccess[iOldPos].contains(sFileName)) {
	  test_unFileAccessControlConfiguration_removeConfig();
	}
}
