/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"

#uses "test/unImportDevice/unImportDeviceTest_helper.ctl"


#uses "unImportDevice.ctl"
#uses "fwDevice/fwDevice.ctl"

const string DEVICES_DRIVER_NUMBER = "3";


//-------------------------------------------------------
void test_unImportDevice_setupSuite() {
  unImportTestDevice_helper_startDriver(DEVICES_DRIVER_NUMBER);
  fwDevice_initialize(); //needed by unImportDevice_checkFile()... dont ask me why
}

//-------------------------------------------------------
void test_unImportDevice_createSoftFrendEnd() {
  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Create.txt";
  string sLogContent;
 
  //import
  importOk = unImportTestDevice_helper_importDevices(sImportFile, DEVICES_DRIVER_NUMBER, sLogContent);

  //check that all is ok 
  bool bIsWarningPresent = strpos(sLogContent, "WARNING") >= 0;
  bool bIsErrorPresent = strpos(sLogContent, "ERROR") >= 0;
  bool bIsImportSuccess = strpos(sLogContent, "file was successfully imported") >= 0;

  assertEqual( true, importOk, "An error occured while importing " + sImportFile + "\n"+sLogContent);
  assertEqual( false, bIsWarningPresent, "There was a warning when importing " + sImportFile + "\n"+sLogContent);
  assertEqual( false, bIsErrorPresent, "There was an error when importing " + sImportFile + "\n"+sLogContent);
  assertEqual( true, bIsImportSuccess, "Import did not complete (log does not contains successful importation statement) "+sImportFile + "\n"+sLogContent);
}

//-------------------------------------------------------
void test_unImportDevice_deleteExistingSoftFrendEnd() {
  //first we make sure to have the front end
  test_unImportDevice_createSoftFrendEnd();

  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_Delete.txt";
  string sLogContent;
 
  //import
  importOk = unImportTestDevice_helper_importDevices(sImportFile, DEVICES_DRIVER_NUMBER, sLogContent);

  //check that all is ok 
  bool bIsWarningPresent = strpos(sLogContent, "WARNING") >= 0;
  bool bIsErrorPresent = strpos(sLogContent, "ERROR") >= 0;
  bool bIsDeleteSuccess = strpos(sLogContent, "file was successfully imported") >= 0;

  assertEqual( true, importOk, "An error occured while importing " + sImportFile + "\n"+sLogContent);
  assertEqual( false, bIsWarningPresent, "There was a warning when deleting " + sImportFile + "\n"+sLogContent);
  assertEqual( false, bIsErrorPresent, "There was an error when deleting " + sImportFile + "\n"+sLogContent);
  assertEqual( true, bIsDeleteSuccess, "Deletion did not complete (log does not contains successful importation statement) "+sImportFile + "\n"+sLogContent);
}

//-------------------------------------------------------
//---UNCORE-173 warn user when front end does not exist
void test_unImportDevice_deleteNonExistingSoftFrendEnd() {
  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_DeleteWrongName.txt";
  bool checkBeforeImporting = true;
  string sLogContent;
 
  //import
  importOk = unImportTestDevice_helper_checkDevices(sImportFile, "SOFT_FE", DEVICES_DRIVER_NUMBER, sLogContent);

  //check that all is ok 
  bool bIsWarningPresent = strpos(sLogContent, "WARNING") >= 0;
  bool bIsErrorPresent = strpos(sLogContent, "ERROR") >= 0;

  assertEqual( true, importOk, "An error occured while importing " + sImportFile + "\n"+sLogContent);
  assertEqual( true, bIsWarningPresent, "Warning is not present indicating that the frontend does not exist " + sImportFile + "\n"+sLogContent);
  assertEqual( false, bIsErrorPresent, "There was an error when deleting " + sImportFile + "\n"+sLogContent);
}

//-------------------------------------------------------
//---UNCORE-173 warn user when front end does not exist
void test_unImportDevice_deleteNonExistingSoftFrendEnd2() {
  bool importOk = true;
  string sImportFile = "test_unImportDevice/wincc_oa_db_file_SoftFE_DeleteWrongName2.txt";
  bool checkBeforeImporting = true;
  string sLogContent;
 
  //import
  importOk = unImportTestDevice_helper_checkDevices(sImportFile, "SOFT_FE", DEVICES_DRIVER_NUMBER, sLogContent);

  //check that all is ok 
  bool bIsWarningPresent = strpos(sLogContent, "WARNING") >= 0;
  bool bIsErrorPresent = strpos(sLogContent, "ERROR") >= 0;

  assertEqual( true, importOk, "An error occured while importing " + sImportFile + "\n"+sLogContent);
  assertEqual( true, bIsWarningPresent, "Warning is not present indicating that the frontend does not exist " + sImportFile + "\n"+sLogContent);
  assertEqual( false, bIsErrorPresent, "There was an error when deleting " + sImportFile + "\n"+sLogContent);
}


