/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"

//------------------------------------
// This routine is optional. It is never called, presence of this routine just tells to
// test runner that tests of this library shall run in UI manager
void test_unMenuConfiguration_alwaysRunInUI()
{
}

//------------------------------------------------------
void test_unMenuConfiguration_setupSuite() {
}

//------------------------------------------------------
void test_unMenuConfiguration_addConfig() {
	
	dyn_string dsRes;
	dyn_float dfRes;
	
	string sShape = "Management";
	string sConf = "insertItemId";
	string sText = "Diagnostic";
	string sType = "item";
	int iType = 0;
	string sItem = "Test_MenuConfig";
	string sFile = "vision/test/unCore/unMenuConfiguration/test_unMenuConfiguration.pnl";
	string sComponent = "testUnMenuConfig";
	string sAccess = "Admin";
	string sConfigLine = sShape + ";" + sConf + ";" + sText + ";" + iType + ";" + sItem + ";" + sFile + ";" + sComponent + ";Access" + sAccess + ";";
	
	dyn_string dsMenuAccess;
	dpGet("_unApplication.menu.menuConfiguration", dsMenuAccess);
	int iOldPos = dynlen(dsMenuAccess);
	
	// Remove test Menu Configuration entry if it already exists
	if(dsMenuAccess[iOldPos].contains(sFile)) {
	  test_unMenuConfiguration_removeConfig();
	  dpGet("_unApplication.menu.menuConfiguration", dsMenuAccess);
	  iOldPos = dynlen(dsMenuAccess);
	}

	ChildPanelOnReturn("vision/test/unCore/unMenuConfiguration/test_unMenuConfiguration_addConfig.xml",
								"testAddConfig",
								makeDynString("$sShape:"+sShape, "$sConf:"+sConf, "$sText:"+sText, "$sType:"+sType, "$sItem:"+sItem, "$sFile:"+sFile, "$sComponent:"+sComponent, "$sAccess:"+sAccess),
								-1, -1,
								dfRes,
								dsRes);

	// Check that a new line has been added to the list of menu configurations
	dpGet("_unApplication.menu.menuConfiguration", dsMenuAccess);
	int iNewPos = dynlen(dsMenuAccess);
	assertEqual(iOldPos+1, iNewPos, "The length of the configuration after adding a new line is incorrect");
	assertEqual(dsMenuAccess[iNewPos], sConfigLine, "New added configuration line is incorrect");
}

//---------------------------------------------------------
void test_unMenuConfiguration_removeConfig() {
	
	dyn_string dsRes;
	dyn_float dfRes;

	string sFile = "vision/test/unCore/unMenuConfiguration/test_unMenuConfiguration.pnl";

	dyn_string dsMenuAccess;
	dpGet("_unApplication.menu.menuConfiguration", dsMenuAccess);
	int iOldPos = dynlen(dsMenuAccess);

	// Add test Menu Configuration entry if it does not exist
	if(!dsMenuAccess[iOldPos].contains(sFile)) {
	  test_unMenuConfiguration_addConfig();
	  dpGet("_unApplication.menu.menuConfiguration", dsMenuAccess);
	  iOldPos = dynlen(dsMenuAccess);
	}

	ChildPanelOnReturn("vision/test/unCore/unMenuConfiguration/test_unMenuConfiguration_removeConfig.xml",
								"testRemoveConfig",
								makeDynString(""),
								-1, -1,
								dfRes,
								dsRes);

	// Check that the last line of the list of menu configurations has been removed
	dpGet("_unApplication.menu.menuConfiguration", dsMenuAccess);
	int iNewPos = dynlen(dsMenuAccess);
	assertEqual(iOldPos-1, iNewPos, "The length of the configuration after removing the last line is incorrect");
}

//------------------------------------------------------
void test_unMenuConfiguration_teardownSuite() {
	dyn_string dsMenuAccess;
	string sFile = "vision/test/unCore/unMenuConfiguration/test_unMenuConfiguration.pnl";
	dpGet("_unApplication.menu.menuConfiguration", dsMenuAccess);
	int iOldPos = dynlen(dsMenuAccess);
	// Remove test Menu Configuration entry if it exists before closing
	// the unMenuConfiguration unit test suite
	if(dsMenuAccess[iOldPos].contains(sFile)) {
	  test_unMenuConfiguration_removeConfig();
	}
}
