/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/


#uses "fwUnitTestComponentAsserts.ctl"
#uses "unCore/unMenuConfigurationExportImport.ctl"

dyn_string originalMenuSettings; //global var to reset the menu as it was at the end of the tests

//------------------------------------------------------
void test_unMenuConfigurationExportImport_setupSuite() {
  dpGet("_unApplication.menu.menuConfiguration", originalMenuSettings);
}

//------------------------------------------------------
void test_unMenuConfigurationExportImport_teardownSuite() {
  dpSetWait("_unApplication.menu.menuConfiguration", originalMenuSettings);
}

//------------------------------------------------------
void test_unMenuConfigurationExportImport_exportToFile_ok() {
  string tmpFile = tmpnam();

  try{
    unMenuConfiguration_exportToFile(tmpFile);
  } catch{
    assert(false, "Exception thrown when exporting to file "+getLastException());
    return;
  }

  dyn_string exceptedContent;
  try{
    dpGet("_unApplication.menu.menuConfiguration", exceptedContent);
     FwException::checkLastError();
  } catch{
    assert(false, "Exception thrown when getting the content of menuConfiguration DP: "+getLastException());
    return;
  }

  string fileContent;
  fileToString(tmpFile, fileContent);

  dyn_string fileContentAsDyn = strsplit(fileContent,"\n");

  assertEqual(exceptedContent, fileContentAsDyn, "Exported file content does not match the content of the menu configuration DP.");
}

//------------------------------------------------------
void test_unMenuConfigurationExportImport_exportImport_ok() {
  string tmpFile = tmpnam();

  try{
    unMenuConfiguration_exportToFile(tmpFile);
  } catch{
    assert(false, "Exception thrown when exporting to file "+getLastException());
    return;
  }

  //set dpe to another value to make sure it is changed when importing
  dpSetWait("_unApplication.menu.menuConfiguration", makeDynString(""));


  try{
    unMenuConfiguration_importFromFile(tmpFile);
  } catch{
    assert(false, "Exception thrown when importing from file "+getLastException());
    return;
  }

  string tmpFile2 = tmpnam();

  try{
    unMenuConfiguration_exportToFile(tmpFile2);
  } catch{
    assert(false, "Exception thrown when exporting to second file "+getLastException());
    return;
  }

  string fileContent;
  fileToString(tmpFile, fileContent);
  string fileContent2;
  fileToString(tmpFile2, fileContent2);

  assertEqual( fileContent, fileContent2, "Exported file does not match the content of imported file");
  assert(fileContent2!="", "Exported file should not be empty");
}

//------------------------------------------------------
void test_unMenuConfigurationExportImport_exportToFile_invalidFile() {
  string tmpFile = "filename/with/slash/can/not/exist";

  try{
    unMenuConfiguration_exportToFile(tmpFile);
  } catch{
    assert(true, "Exception was thrown as expected when exporting to file "+tmpFile);
    return;
  }

  assert(false, "An exception should have been thrown when exporting to file "+tmpFile);
}

//------------------------------------------------------
void test_unMenuConfigurationExportImport_importFromFile_invalidFile() {
  string tmpFile = "filename/with/slash/can/not/exist";

  try{
    unMenuConfiguration_importFromFile(tmpFile);
  } catch{
    assert(true, "Exception was thrown as expected when importing from inexisting file "+tmpFile);
    return;
  }

  assert(false, "An exception should have been thrown when importing from inexisting file "+tmpFile);
}

//------------------------------------------------------
void test_unMenuConfigurationExportImport_exportToFile_contentDifferent() {
  string tmpFile = tmpnam();

  try{
    unMenuConfiguration_exportToFile(tmpFile);
  } catch{
    assert(false, "Exception thrown when exporting to file "+getLastException());
    return;
  }

  dyn_string exceptedContent=makeDynString("Hellow world!"); //leave it random (do not get the value from DP)

  string fileContent;
  fileToString(tmpFile, fileContent);

  dyn_string fileContentAsDyn = strsplit(fileContent,"\n");

  assert(exceptedContent != fileContentAsDyn, "Content from file and random dyn_string should have been different.");
}