/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "fwUnitTestComponentAsserts.ctl"

const string TEST_DP_NAME = "testArchive";
const string TEST_DPT_NAME = "ExampleDP_Int";
const string TEST_DP_ALIAS = "testArchive_DeviceAlias";

//------------------------------------
void test_unSetDeviceDPEConfigs_setupSuite() {
	if (dpExists(TEST_DP_NAME)){
		dpDelete(TEST_DP_NAME);
	}

          dpCreate(TEST_DP_NAME, TEST_DPT_NAME);
          dpSetAlias(TEST_DP_NAME + ".", TEST_DP_ALIAS);
}

//------------------------------------
// This routine is optional. It is never called, presence of this routine just tells to
// test runner that tests of this library shall run in UI manager
void test_unSetDeviceDPEConfigs_alwaysRunInUI()
{
}


//------------------------------------
void test_unSetDeviceDPEConfigs_addArchiveConfig() {

	dyn_string dsRes;
	dyn_float dfRes;

 	ChildPanelOnReturn("vision/test/unCore/unSetDeviceDPEConfigs/test_unSetDeviceDPEConfigs_addArchive.xml",
								"testAddArchive",
								makeDynString("$sTestAlias:"+TEST_DP_ALIAS, "$sTestDpe:."),
								-1, -1,
								dfRes,
								dsRes);

	//	Check now that the test dp has an archive config
	int iType;
	bool bArchive;
	dpGet(TEST_DP_NAME+".:_archive.._type", iType);
	assertEqual( iType, DPCONFIG_DB_ARCHIVEINFO, TEST_DP_NAME+" should have an archive config but does not: "+iType);
	dpGet(TEST_DP_NAME+".:_archive.._archive", bArchive);
	assertEqual( bArchive, true, TEST_DP_NAME+" should have its archive config active but does not: "+bArchive);
}


//------------------------------------
void test_unSetDeviceDPEConfigs_deleteArchiveConfig() {

	//first make sure that we have archive
	test_unSetDeviceDPEConfigs_addArchiveConfig();

	//Now launch the panel simulating the archive deletion
	dyn_string dsRes;
	dyn_float dfRes;

 	ChildPanelOnReturn("vision/test/unCore/unSetDeviceDPEConfigs/test_unSetDeviceDPEConfigs_deleteArchive.xml",
								"testAddArchive",
								makeDynString("$sTestAlias:"+TEST_DP_ALIAS, "$sTestDpe:."),
								-1, -1,
								dfRes,
								dsRes);

	//Check now that the test dp has an archive config
	int iType;
	bool bArchive;
	dpGet(TEST_DP_NAME+".:_archive.._type", iType);
	assertEqual( iType, DPCONFIG_NONE, TEST_DP_NAME+" should not have an archive config but does: "+iType);
}

//------------------------------------
void test_unSetDeviceDPEConfigs_teardownSuite() {
  if (dpExists(TEST_DP_NAME)) {
    dpDelete(TEST_DP_NAME);
  }
}
