/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

/**@name LIBRARY: unSetDeviceDPEConfigs.ctl

@author: Herve Milcent (EN-ICE)

Creation Date: 06/02/2009

Modification History:

version 1.1

External Functions:
  . unSetDeviceDPEConfigs_initialize
  . unSetDeviceDPEConfigs_displayCallBack
  . unSetDeviceDPEConfigs_handleEvent

Internal Functions:

Purpose:
This library contains the functions to be used in the panel unSetDeviceDPEConfigs.pnl.

Usage: Public

PVSS manager usage: UI (WCCOAui)

Constraints:
  . use the unDeviceTree.pnl widget
  . PVSS version: 3.6
  . PVSS manager type: WCCOAui
  . operating system: WXP, Linux.
  . distributed system: yes.
*/

//@{
private global dyn_string g_dsSelectedKey, g_dsCompositeLabel;
//--------------------------------------------------------------------------------------------------------------------------------
//unSetDeviceDPEConfigs_initialize
/**
Purpose: Initialise function.

Parameters:
	- sIdentifier: string, input, device identifier
	- sDpType, string, input, object
	- exceptionInfo: dyn_string, output, details of any exceptions are returned here

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.1
	. operating system: WXP.
	. PVSS manager type: WCCOAui
	. distributed system: yes.

@reviewed 2018-09-17 @whitelisted{LiteralString}
*/
unSetDeviceDPEConfigs_initialize()
{
//  DebugTN("unSetDeviceDPEConfigs_initialize");
}

//--------------------------------------------------------------------------------------------------------------------------------
//unSetDeviceDPEConfigs_displayCallBack
/**
Purpose: Callback function trigger from the unDeviceTree panel widget

Parameters:
  - sCommandDp: string, input, callback DP
  - iCommand, int, input, the command to execute
  - sSystemNameDp: string, input, callback DP
  - sSystemName, string, input, the system name to use for the iCommand

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.1
	. operating system: WXP.
	. PVSS manager type: WCCOAui
	. distributed system: yes.

@reviewed 2018-09-17 @whitelisted{LiteralString}
*/
unSetDeviceDPEConfigs_displayCallBack(string sCommandDp, int iCommand, string sSystemNameDp, string sSystemName)
{
//DebugTN("unSetDeviceDPEConfigs_displayCallBack start");
  unTreeDeviceOverview_displayCallBackTreeDeviceWidget(sCommandDp, iCommand, sSystemNameDp, sSystemName);
//DebugTN("unSetDeviceDPEConfigs_displayCallBack end");
}

//--------------------------------------------------------------------------------------------------------------------------------
//unSetDeviceDPEConfigs_handleEvent
/**
Purpose: Function triggerd by the unDeviceTree widget to handle the event from the device tree

Parameters:
  - sCommand: string, input, the command to execute
  - dsSelectedKey, dyn_string, input, the key of the selected nodes of the device tree
  - dsCompositeLabel: dyn_string, input, the composite label of the selected nodes of the device tree

Usage: Public

PVSS manager usage: NG, NV

Constraints:
	. PVSS version: 3.1
	. operating system: WXP.
	. PVSS manager type: WCCOAui
	. distributed system: yes.

@reviewed 2018-09-17 @whitelisted{LiteralString}
*/
unSetDeviceDPEConfigs_handleEvent(string sCommand, dyn_string dsSelectedKey, dyn_string dsCompositeLabel)
{
  switch(sCommand) {
      case DEVUN_TREEDISPLAY_NODELEFTCLICK:
        unSetDeviceDPEConfigs_setKeyAndLabel(dsSelectedKey, dsCompositeLabel);
        triggerCommand.text = sCommand;
//  DebugTN("unSetDeviceDPEConfigs_handleEvent: nodeleft", sCommand, sSelectedKey, sCompositeLabel);
        break;
      default:
        break;
    }
}

//--------------------------------------------------------------------------------------------------------------------------------
//unSetDeviceDPEConfigs_getKeyAndLabel
/**
Purpose: Returns the list of selected keys and composite labels for a set of devices

Parameters:
  - dsSelectedKey, dyn_string, output, the key of the selected nodes of the device tree
  - dsCompositeLabel: dyn_string, ouput, the composite label of the selected nodes of the device tree
*/

unSetDeviceDPEConfigs_getKeyAndLabel(dyn_string &dsSelectedKey, dyn_string &dsCompositeLabel)
{
  dsSelectedKey = g_dsSelectedKey;
  dsCompositeLabel = g_dsCompositeLabel;
}

//--------------------------------------------------------------------------------------------------------------------------------
//unSetDeviceDPEConfigs_setKeyAndLabel
/**
Purpose: Sets the list of selected keys and composite labels for a set of devices

Parameters:
  - dsSelectedKey, dyn_string, input, the key of the selected nodes of the device tree
  - dsCompositeLabel: dyn_string, input, the composite label of the selected nodes of the device tree
*/

unSetDeviceDPEConfigs_setKeyAndLabel(dyn_string dsSelectedKey, dyn_string dsCompositeLabel)
{
  g_dsSelectedKey = dsSelectedKey;
  g_dsCompositeLabel = dsCompositeLabel;
}

//--------------------------------------------------------------------------------------------------------------------------------

//@}
